/*
   MODULE NAME        CICS_ESI.H

   DESCRIPTIVE NAME   CICS Password Management Interface

 Licensed Materials - Property of IBM  
  
 "Restricted Materials of IBM"  
  
 5724-I81,5725-B65 
  
 (C) Copyright IBM Corp. 1998, 2014  
  
 US Government Users Restricted Rights - Use, duplication or  
 disclosure restricted by GSA ADP Schedule Contract with  
 IBM Corp.  
   
   Status: Version 9 Release 2 

   NOTES :-

   This header file is provided with the CICS Transaction Gateway. The
   Gateway is available for a number of different operating
   environments and to ensure the correct sections of this header are
   included one of the following constants must be defined prior to
   inclusion:

     CICS_W32  -  if building a Windows application
     CICS_AIX  -  if building an AIX application
     CICS_SOL  -  if building a Solaris application
     CICS_HPUX -  if building an HPUX application
     CICS_LNX  -  if building a Linux application
     CICS_HPIT and CICS_HPUX -
                  if building an HPUX application on Itanium 
*/

#ifndef _CICS_ESI_H
#define _CICS_ESI_H

#include <cicstype.h>
#include <cics_eci.h>

#ifdef __cplusplus
  extern "C" {
#endif


#if defined(CICS_AIX)
#pragma options align=packed
#elif defined(CICS_HPUX)
#pragma pack 1
#else
#pragma pack(1)
#endif

#define CICS_ESI_USERID_MAX       10
#define CICS_ESI_PASSWORD_MAX     10
#define CICS_ESI_SYSTEM_MAX       8

/*
   ESI Data Structures
*/

typedef struct
{
   cics_ushort_t   Year;
   cics_ushort_t   Month;              /* Valid values in range 1 to 12.      */
   cics_ushort_t   Day;                /* Valid values in range 1 to 31.      */
} CICS_EsiDate_t;

typedef struct
{
   cics_ushort_t   Hours;              /* Valid values in range 0 to 23.      */
   cics_ushort_t   Minutes;            /* Valid values in range 0 to 59.      */
   cics_ushort_t   Seconds;            /* Valid values in range 0 to 59.      */
   cics_ushort_t   Hundredths;         /* Valid values in range 0 to 99.      */
} CICS_EsiTime_t;

typedef struct
{
   CICS_EsiDate_t  LastVerifiedDate;
   CICS_EsiTime_t  LastVerifiedTime;
   CICS_EsiDate_t  ExpiryDate;
   CICS_EsiTime_t  ExpiryTime;
   CICS_EsiDate_t  LastAccessDate;
   CICS_EsiTime_t  LastAccessTime;
   cics_ushort_t   InvalidCount;
} CICS_EsiDetails_t;


/*
   Function prototypes for ESI.
*/

cics_sshort_t CICSCALL CICS_VerifyPassword(cics_char_t   CICSPTR *UserId,
                                           cics_char_t   CICSPTR *Password,
                                           cics_char_t   CICSPTR *System,
                                           CICS_EsiDetails_t CICSPTR *Details);

cics_sshort_t CICSCALL CICS_ChangePassword(cics_char_t   CICSPTR *UserId,
                                           cics_char_t   CICSPTR *OldPassword,
                                           cics_char_t   CICSPTR *NewPassword,
                                           cics_char_t   CICSPTR *System,
                                           CICS_EsiDetails_t CICSPTR *Details);

cics_sshort_t CICSCALL CICS_SetDefaultSecurity(cics_char_t   CICSPTR *UserId,
                                               cics_char_t   CICSPTR *Password,
                                               cics_char_t   CICSPTR *System);

/*
   ESI return codes.
*/
#define CICS_ESI_NO_ERROR                ECI_NO_ERROR
#define CICS_ESI_ERR_NO_CICS             ECI_ERR_NO_CICS
#define CICS_ESI_ERR_CICS_DIED           ECI_ERR_CICS_DIED
#define CICS_ESI_ERR_SYSTEM_ERROR        ECI_ERR_SYSTEM_ERROR
#define CICS_ESI_ERR_RESOURCE_SHORTAGE   ECI_ERR_RESOURCE_SHORTAGE
#define CICS_ESI_ERR_NO_SESSIONS         ECI_ERR_NO_SESSIONS
#define CICS_ESI_ERR_UNKNOWN_SERVER      ECI_ERR_UNKNOWN_SERVER
#define CICS_ESI_ERR_CALL_FROM_CALLBACK  ECI_ERR_CALL_FROM_CALLBACK
#define CICS_ESI_ERR_SECURITY_ERROR      ECI_ERR_SECURITY_ERROR
#define CICS_ESI_ERR_MAX_SYSTEMS         ECI_ERR_MAX_SYSTEMS
#define CICS_ESI_ERR_MAX_SESSIONS        ECI_ERR_MAX_SESSIONS
#define CICS_ESI_ERR_NULL_USERID         -100
#define CICS_ESI_ERR_NULL_OLD_PASSWORD   -101
#define CICS_ESI_ERR_NULL_NEW_PASSWORD   -102
#define CICS_ESI_ERR_PEM_NOT_SUPPORTED   -103
#define CICS_ESI_ERR_PEM_NOT_ACTIVE      -104
#define CICS_ESI_ERR_PASSWORD_EXPIRED    -105
#define CICS_ESI_ERR_PASSWORD_INVALID    -106
#define CICS_ESI_ERR_PASSWORD_REJECTED   -107
#define CICS_ESI_ERR_USERID_INVALID      -108
#define CICS_ESI_ERR_NULL_PASSWORD       CICS_ESI_ERR_NULL_OLD_PASSWORD

#if defined(CICS_AIX)
#pragma options align=reset
#elif defined(CICS_HPUX) && defined(__cplusplus)
#pragma pack
#elif defined(CICS_HPUX) && defined(CICS_HPIT)
#pragma pack
#elif defined(CICS_HPUX)
#pragma HP_ALIGN HPUX_NATURAL
#else
#pragma pack()
#endif

#ifdef __cplusplus
  }
#endif

#endif /* _CICS_ESI_H */
