/* File Name(s)  : epia1.c

   Product       : CICS Transaction Gateway

   Description   : This sample shows an advanced form of using the CICS TG C
                   API. It queries the Client daemon for a list of available  
                   servers and then creates a number of threads per server.
                   Each thread runs a number of EPI transactions on that
                   server, and works out the average response time per server.

   Prerequisites:  This samples requires the CICS Transaction EP01 to be
                   installed on your CICS server for operation.

 Licensed Materials - Property of IBM  
  
 5724-I81,5725-B65 
  
 (C) Copyright IBM Corp. 1999, 2014 All Rights Reserved.  
  
 US Government Users Restricted Rights - Use, duplication or  
 disclosure restricted by GSA ADP Schedule Contract with  
 IBM Corp.  
   
   Status: Version 9 Release 2 

                   The following code is sample code created by IBM
                   Corporation.  This sample code is not part of any standard
                   IBM product and is provided to you solely for the purpose
                   of assisting you in the development of your applications.
                   The code is provided 'AS IS', without warranty or
                   condition of any kind.  IBM shall not be liable for any
                   damages arising out of your use of the sample code, even
                   if IBM has been advised of the possibility of such damages. */


/* Include files */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <stddef.h>
#include <ctype.h>

#include <cics_epi.h>
#include <cclcalls.h>
#include <dfhaid.h>

#define BUFFER_SIZE 4096

#define copyright_notice copyright4cepia1
char version4cepia1[] =
"\nCICS Transaction Gateway"
" Version 9.2.0.3"
" Build Level c920-20180312-1735" ;

char copyright_notice[] =
"\n"
"Licensed Materials - Property of IBM. "
"\n"
"5724-I81,5725-B65 "
"\n"
"(c) Copyright IBM Corp. 1999, 2014 All Rights Reserved. "
"\n"
"US Government Users Restricted Rights - Use, duplication or "
"disclosure restricted by GSA ADP Schedule Contract with "
"IBM Corp."
"\n";

/* Structures, defines and macro definitions */

typedef struct {
   char               AIDKey;
   char               Byte0;
   char               Byte1;
   char               TransactionCode[4];
   char               Data[BUFFER_SIZE];
} DataStream;

struct RunInfo
{
   int TotalThreads;
   int NumCompleteOK;
   int NumCompleteError;
};

#define MAX_SYSTEMS   20

#define CCLLOG(A)        printf("%d (%d,%d - %d) => %s\n", QueryMyThreadId(), Idx, \
                                 Thread, termIndex, A);                            \
                         fflush(stdout);
#define CCLLOG2(A,B,C,D) printf("%d (%d,%d - %d) => ", QueryMyThreadId(), Idx,     \
                                               Thread, termIndex);                 \
                         printf(A,B,C,D);                                          \
                         printf("\n");                                             \
                         fflush(stdout);

#define SYNC_OPTION      "SYNC"
#define ASYNC_OPTION     "ASYNC"

#if defined(CICS_W32)
#define DEBUG_OPTION     "/DEBUG"
#else
#define DEBUG_OPTION     "-DEBUG"
#endif

/* Global Variables - If accessed by multiple threads, must be protected by
                      a mutex semaphore */

cics_ushort_t serverCount = MAX_SYSTEMS;/*Actual number of servers in ini file         */
CICS_EpiSystem_t servers[MAX_SYSTEMS];  /*Server list for use in CICS_EpiListSystems().*/
float    *average;                      /*Average time per call so far                 */
CclEvent  StartTest,                    /*Event flagged to start the test              */
FinishTest,                             /*Event flagged by last server thread          */
StartedThread;                          /*Event flagged as a server thread starts up   */
CclEvent *Callback;                     /*Array of events for async calls              */


/* Saved copies of initial args */

int       ThreadsPerServer = 5;         /* Threads per server (Default: 1)             */
int       CallsPerThread   = 10;        /* Number of calls per thread (Default: 10)    */
CclBool   Debug            = CCLFALSE;  /* Display Debug information                   */
CclBool   AsyncCalls       = CCLFALSE;  /* Are we sync/async                           */

struct RunInfo Progress; /* Tracks threads started/completed */
CclMutex ProgressLock; /* Protects update of Progress object */


/* Function prototypes */

void Response(int Rc, CclThreadId Tid);
void DumpDataStream(cics_char_t *Start, cics_ushort_t Size);
char *EndReason(CICS_EpiEnd_t endReason);
char *StrToupper(char *str);

#if defined(CICS_W32)
void __cdecl EPILoop(void *IdxPtr);
#else
void EPILoop(void *IdxPtr);
#endif

void CICSEXIT CallBackProc (cics_ushort_t AIdx);
short WaitForCICSEvent(short termIndex, int Idx, int Thread, short *RealRc, CICS_EpiDetails_t *termDetails);
char* EventType(int event);
void AbortThread(void);


/* Main Function */

int main(int argc, char *argv[]){
   short rc;
   struct Parms {
      int i;
      int j;
   } Parm;
   float RunningTotal;
   int   Size;
   char  TermInput[100];

   /* Display Banner and syntax of program */

   printf("CICS Transaction Gateway Advanced EPI Sample\n\n");
   printf("SYNTAX : \n");
   printf("  epia1 <Threads> <Calls> <%s|%s> [%s]\n", SYNC_OPTION, ASYNC_OPTION, DEBUG_OPTION);
   printf("    where \n");
   printf("  Threads       = Number of threads to run per server (Default=5)\n");
   printf("  Calls         = Number of EPI transactions to make per thread (Default=10)\n");
   printf("  SYNC or ASYNC = Do (a)synchronous event processing (Default=SYNC)\n");
   printf("  %s     = Show all messages (Default is off)\n", DEBUG_OPTION);
   printf("\n");


   /* Parsing of the args & perform error checking */

   if (argc > 1) {
      if(atol(argv[1]) < 1) {
         printf("ERROR: \"%s\" %s", argv[1], " is not a valid choice, threads per server needs to be above 0");
         fflush(stdout);
         return -1;
      }
      else ThreadsPerServer = atol(argv[1]);
   }
   if (argc > 2) {
     if(strcmp(argv[2], "0") == 0 || atol(argv[2]) > 0) CallsPerThread   = atol(argv[2]);
     else {
        printf("ERROR: \"%s\" %s", argv[2], " is not a valid choice, EPI calls per thread needs to be 0 or above");
        fflush(stdout);
        return -1;
     }
   }
   if (argc > 3) {
      if (strcmp(StrToupper(argv[3]), ASYNC_OPTION) == 0) AsyncCalls = CCLTRUE;
     else if(strcmp(StrToupper(argv[3]), SYNC_OPTION) == 0) AsyncCalls = CCLFALSE;
     else {
        printf("ERROR: \"%s\" is not a valid choice", argv[3]);
        fflush(stdout);
        return -1;
     }
   }
   if (argc > 4) {
      if(strcmp(StrToupper(argv[4]), DEBUG_OPTION) == 0) Debug = CCLTRUE;
      else {
        printf("ERROR: \"%s\" is not a valid choice", argv[4]);
        fflush(stdout);
        return -1;
     }
   }


   /* Initialize to EPI */

   rc = CICS_EpiInitialize(CICS_EPI_VERSION_200);
   if (rc != CICS_EPI_NORMAL) {
      printf("MAIN : Failed to initialize with EPI - aborting\n");
      Response(rc, 0);
      return 1;
   }


   /* Ask the Client daemon for a list of servers to connect to */

   rc = CICS_EpiListSystems(NULL,&serverCount,servers);
   if (rc != CICS_EPI_NORMAL) {
      Response(rc, 0);
      if (rc != CICS_EPI_ERR_MORE_SYSTEMS) return 1;
   }


   /* Display the parameters which will be used */

   printf("Running with: \n");
   printf("%d servers\n", serverCount);
   printf("%d threads per server\n", ThreadsPerServer);
   printf("%d EPI transactions per thread\n", CallsPerThread);

   if (AsyncCalls == CCLTRUE) {
      printf("Event processing is asynchronous\n");
   } else {
      printf("Event processing is synchronous\n");
   }


   /* Create a Mutex and some Event semaphores */

   if (CreateAnEvent(NULL, &StartedThread) != CCLTRUE) {
      printf("MAIN : Failed to create thread started event - aborting process\n");
      fflush(stdout);
      return 1;
   }
   if (CreateAMutex(NULL, &ProgressLock) != CCLTRUE) {
      printf("MAIN : Failed to create main mutex - aborting process\n");
      fflush(stdout);
      return 1;
   }
   if (CreateAnEvent(NULL, &StartTest) != CCLTRUE) {
      printf("MAIN : Failed to create StartTest event - aborting process\n");
      fflush(stdout);
      return 1;
   }
   if (CreateAnEvent(NULL, &FinishTest) != CCLTRUE) {
      printf("MAIN : Failed to create FinishTest event - aborting process\n");
      fflush(stdout);
      return 1;
   }


   /* Malloc some storage for the averages and events used if async */

   /* Malloc storage for the averages */

   Size = sizeof(float) * serverCount * ThreadsPerServer;
   average = malloc(Size);
   if (average == CCLNULL) {
      printf("MAIN : Failed to malloc storage - aborting process\n");
      fflush(stdout);
      return 1;
   }

   /* Malloc storage for async calls */

   if (AsyncCalls == CCLTRUE) {
      Size = sizeof(CclEvent) * serverCount * ThreadsPerServer;
      Callback = malloc(Size);
      if (Callback == CCLNULL) {
         printf("MAIN : Failed to malloc storage for events - aborting process\n");
         fflush(stdout);
         return 1;
      } else {
         memset(Callback, 0x00, Size);
      }
   }

   Progress.NumCompleteError = 0;
   Progress.NumCompleteOK = 0;
   Progress.TotalThreads = serverCount * ThreadsPerServer;


   /* Start the required number of threads for each server */

   for (Parm.i=0; Parm.i<serverCount; Parm.i++) {
      for (Parm.j=0; Parm.j<ThreadsPerServer; Parm.j++) {

         /* Start a new thread */

         if (StartAThread( EPILoop, &Parm ) != CCLTRUE) {
            printf("MAIN : Failed to create thread %d/%d - aborting process\n", Parm.i, Parm.j);
            fflush(stdout);
            return 1;
         }

         /* Wait for it to completely start */

         if (Debug == CCLTRUE) printf("MAIN : Waiting for new thread to start\n");
         if (WaitOnAnEvent( &StartedThread ) != CCLTRUE) {
            printf("MAIN : Failed to wait for thread %d/%d to start - aborting process\n", Parm.i, Parm.j);
            fflush(stdout);
            return 1;
         }

         /* Reset the event so it can be reused */

         if (ResetAnEvent( &StartedThread ) != CCLTRUE) {
            printf("MAIN : Failed to wait for thread %d/%d to start - aborting process\n", Parm.i, Parm.j);
            fflush(stdout);
            return 1;
         }
      }
   }
   printf("MAIN : Threads all started on %d servers - starting test\n", serverCount);


   /* All threads have started and are waiting to go. Post the StartTest event*/

   if (FlagAnEvent( &StartTest ) != CCLTRUE) {
      printf("MAIN : Failed to post the StartTest flag - aborting process\n");
      fflush(stdout);
      return 1;
   }


   /* Give the test a second to complete and then wait on the finish event */

   WaitInMillisecs(1000);

   if (WaitOnAnEvent( &FinishTest ) != CCLTRUE) {
      printf("MAIN : Failed to wait on the FinishTest flag - aborting process\n");
      fflush(stdout);
      return 1;
   }


   /* Display the tests results */

   printf("MAIN : Threads all finished - results follow:\n");
   if (Progress.NumCompleteError > 0) {
      printf("MAIN : %d of %d threads completed in error, results may be affected\n", Progress.NumCompleteError, Progress.TotalThreads);
   }

   for (Parm.i=0; Parm.i<serverCount; Parm.i++) {
      RunningTotal = 0.0;
      for (Parm.j=0; Parm.j<ThreadsPerServer; Parm.j++) {
         if (Debug == CCLTRUE) printf("Server : %s, thread %d average time in seconds %f\n",
                                      servers[Parm.i].SystemName,
                                      Parm.j,
                                      ((float) average[(Parm.i*ThreadsPerServer)+Parm.j])/1000.0);
         RunningTotal = RunningTotal + average[(Parm.i*ThreadsPerServer)+Parm.j];
      }
      printf("Server : %s, overall average time in seconds %f\n",
             servers[Parm.i].SystemName,
             RunningTotal / (1000.0 * (float) ThreadsPerServer));
   }

   printf("Press enter to terminate\n");
   fgets(TermInput, 100, stdin);


   /* Clean up malloced storage and created events/mutexes */

   free(average);
   if (AsyncCalls == CCLTRUE) {
      free(Callback);
   }
   DeleteAnEvent( &StartTest );
   DeleteAnEvent( &FinishTest );
   DeleteAnEvent( &StartedThread );
   DeleteAMutex(  &ProgressLock );

   /* Initialize to EPI */

   rc = CICS_EpiTerminate();
   if (rc != CICS_EPI_NORMAL) {
      printf("MAIN : Failed to terminate EPI - aborting process\n");
      Response(rc, 0);
      return 1;
   }

   fflush(stdout);
   return 0;
}

/* Function started once running on each new thread */

void EPILoop(void *IdxPtr) {
   short rc;                                      /*Return code                                    */
   char CallBackEvt[20];                          /*Name for the callback event                    */
   CICS_EpiDetails_t    termDetails;              /*Returned terminal details                      */
   cics_ushort_t        termIndex = 0xFFFF;       /*Returned terminal index                        */
   CICS_EpiAttributes_t termAttributes;           /*Terminal attributes used for the install       */
   char UserID[CICS_EPI_USERID_MAX] = "SYSAD";    /*UserID                                         */
   char PassWd[CICS_EPI_PASSWORD_MAX] = "SYSAD";  /*Password                                       */
   CclBool AllFinished = CCLFALSE;                /*Set true if this is the last thread to finish  */

   DataStream           epiData;
   void                *dataPointer;

   struct Parms {
      int IdxOfServer;
      int ThreadPerServer;
   } *Parm;

   int Idx, Thread, AIdx;
   int i   = 0;

   unsigned long TimeBefore, TimeAfter;


   /* Extract information from main thread which will change */

   Parm          = (struct Parms *) IdxPtr;
   Idx           = Parm->IdxOfServer;
   Thread        = Parm->ThreadPerServer;
   AIdx          = (Idx * ThreadsPerServer) + Thread;
   average[AIdx] = 0.0;

   if (Debug == CCLTRUE) CCLLOG("Thread started");


   /* Inform main thread we have started and obtained all information we need */

   if (FlagAnEvent( &StartedThread) != CCLTRUE) {
      CCLLOG("Failed to inform main thread via StartedThread - aborting process");
      exit(2); /* Fatal error */
   }


   /* Wait for all other threads and for the test to start */

   if (WaitOnAnEvent( &StartTest ) != CCLTRUE) {
      CCLLOG("Failed to wait on the StartTest flag - aborting thread");
      AbortThread();
      EndMyThread();
      return;
   }

   if (Debug == CCLTRUE) CCLLOG("Running the test");


   /* Install the terminal we wish to run the transaction on. Do this
      as a signon capable terminal. */

   if (Debug == CCLTRUE) CCLLOG("Adding a terminal");

   memset(&termAttributes, 0x00, sizeof(termAttributes));
   if (AsyncCalls == CCLTRUE) {
      termAttributes.EpiAddType = CICS_EPI_ADD_ASYNC;
   } else {
      termAttributes.EpiAddType = CICS_EPI_ADD_SYNC;
   }
   termAttributes.SignonCapability = CICS_EPI_SIGNON_INCAPABLE;
   memcpy(&termAttributes.UserId,       UserID,       CICS_EPI_USERID_MAX);
   memcpy(&termAttributes.Password,     PassWd,       CICS_EPI_PASSWORD_MAX);


   /* If async calls, do the install asynchronously with a callback routine,
      otherwise do it without a callback routine */

   if (AsyncCalls == CCLTRUE) {
      /* Cast needed for HP-UX */
      rc = CICS_EpiAddExTerminal( servers[Idx].SystemName,
                                  NULL, NULL,
                                  (CICS_EpiNotify_t)&CallBackProc,
                                  &termDetails,
                                  &termIndex,
                                  &termAttributes);
   } else {
      rc = CICS_EpiAddExTerminal( servers[Idx].SystemName,
                                  NULL, NULL,
                                  NULL,
                                  &termDetails,
                                  &termIndex,
                                  &termAttributes);
   }

   /* If the AddTerminal call failed, abort otherwise for async installs we
      need to wait for the callback to be driven. */

   if (rc != CICS_EPI_NORMAL) {
      CCLLOG("CICS_EpiAddExTerminal failed - aborting thread");
      Response(rc, QueryMyThreadId());
      AbortThread();
      EndMyThread();
      return;
   }

   if (AsyncCalls == CCLTRUE) {
      sprintf(CallBackEvt, "Callback%d", termIndex);
      if (CreateAnEvent(CallBackEvt, &Callback[termIndex]) != CCLTRUE) {
         CCLLOG("Failed to create callback event - aborting thread");
         AbortThread();
         EndMyThread();
         return;

      }
      if (WaitForCICSEvent(termIndex, Idx, Thread, &rc, &termDetails) != CICS_EPI_EVENT_ADD_TERM) {
         CCLLOG("CICS_EpiAddExTerminal did not return EVENT_ADD_TERM - aborting thread");
         AbortThread();
         EndMyThread();
         return;
      }

      if (rc != CICS_EPI_NORMAL) {
         CCLLOG("CICS_EpiAddExTerminal failed - aborting thread");
         Response(rc, QueryMyThreadId());
         AbortThread();
         EndMyThread();
      }

   }

   if (Debug == CCLTRUE) {
      CCLLOG2("CICS_EpiAddExTerminal call, added idx %d to server %s return code %d",
              termIndex, servers[Idx].SystemName, rc);

      /*Output some information about the terminal.*/

      CCLLOG2("   Connected on terminal: %s, dimensions %d x %d\n", termDetails.TermId,
              termDetails.NumLines, termDetails.NumColumns);
   }


   /* Loop running the transaction for the specified number of times */

   for (i = 0; i<CallsPerThread; i++) {


      /* Set up the 3270 datastream for the StartTran call */

      epiData.AIDKey = 0x27;
      epiData.Byte0  = 0x20;
      epiData.Byte1  = 0x20;
      memcpy (epiData.TransactionCode, "EP01", 4);
      memset (epiData.Data, 0, BUFFER_SIZE);
      dataPointer = &epiData.AIDKey;


      /* Start the transaction and note the start time */

      TimeBefore = CurTimeInMilliSecs();

      if (Debug == CCLTRUE) CCLLOG("Starting EP01");

      rc = CICS_EpiStartTran(termIndex, NULL, dataPointer, offsetof(DataStream, Data));
      if (rc != CICS_EPI_NORMAL) {
         CCLLOG("CICS_EpiStartTran failed - aborting thread");
         Response(rc, QueryMyThreadId());
         AbortThread();
         EndMyThread();
         return;
      }


      /* Wait for a send event indicating the transaction has sent data to
         the screen */

      if (Debug == CCLTRUE) CCLLOG("Waiting for SEND event");
      if (WaitForCICSEvent(termIndex, Idx, Thread, &rc, NULL) != CICS_EPI_EVENT_SEND) {
         CCLLOG("CICS_EpiStartTran did not return EVENT_SEND - aborting thread");
         AbortThread();
         EndMyThread();
         return;
      }

      if (rc != CICS_EPI_NORMAL) {
         CCLLOG("CICS_EpiStartTran failed - aborting thread");
         Response(rc, QueryMyThreadId());
         AbortThread();
         EndMyThread();
      }


      /* Wait for an ENDTRAN event indicating the transaction is complete */

      if (Debug == CCLTRUE) CCLLOG("Waiting for END_TRAN event");
      if (WaitForCICSEvent(termIndex, Idx, Thread, &rc, NULL) != CICS_EPI_EVENT_END_TRAN) {
         CCLLOG("CICS_EpiStartTran did not return EVENT_ENDTRAN - aborting thread");
         AbortThread();
         EndMyThread();
         return;
      }

      if (rc != CICS_EPI_NORMAL) {
         CCLLOG("CICS_EpiStartTran failed - aborting thread");
         Response(rc, QueryMyThreadId());
         AbortThread();
         EndMyThread();
      }


      /* Save the finish time and calculate the transaction time (not
         including the time taken to install/deinstall the terminal) */

      TimeAfter = CurTimeInMilliSecs();

      if (rc == CICS_EPI_NORMAL) {
         average[AIdx] = ((average[AIdx] * (i)) + (TimeAfter-TimeBefore)) / (i+1);
      } else {
         Response(rc, QueryMyThreadId());
         AbortThread();
         EndMyThread();
         return;
      }
   }


   /* Delete the terminal and wait for the END_TERM event */

   if (Debug == CCLTRUE) CCLLOG("Deleting terminal");
   rc = CICS_EpiDelTerminal(termIndex);
   if (rc != CICS_EPI_NORMAL) {
      Response(rc, QueryMyThreadId());
      AbortThread();
      EndMyThread();
      return;
   }

   if (Debug == CCLTRUE) CCLLOG("Waiting for END_TERM event");
   if (WaitForCICSEvent(termIndex, Idx, Thread, &rc, NULL) != CICS_EPI_EVENT_END_TERM) {
      CCLLOG("CICS_EpiDelTerminal did not return EVENT_ENDTERM - aborting thread");
      AbortThread();
      EndMyThread();
      return;
   }

   if (rc != CICS_EPI_NORMAL) {
      CCLLOG("CICS_EpiDelTerminal failed - aborting thread");
      Response(rc, QueryMyThreadId());
      AbortThread();
      EndMyThread();
   }

   if (Debug == CCLTRUE) CCLLOG("Deleted terminal");


   /* Finish the test -If we are the last server thread, start the main thread
      NOTE: since this is a global variable, must protect access from other
      threads with a mutex semaphore */

   if (GetExclusiveAccess( &ProgressLock ) != CCLTRUE) {
      CCLLOG("Failed to get the exclusive access - aborting thread");
      AbortThread();
      EndMyThread();
      return;
   }

   Progress.NumCompleteOK++;

   if (Debug) {
      CCLLOG2("Completed threads now %d because of terminal %d, %s",
           (Progress.NumCompleteOK + Progress.NumCompleteError), termIndex, "");
   }

   if (Progress.NumCompleteOK + Progress.NumCompleteError == Progress.TotalThreads) {
      AllFinished = CCLTRUE;
   }

   if (ReleaseExclusiveAccess( &ProgressLock ) != CCLTRUE) {
      CCLLOG("Failed to release the exclusive access - aborting process");
      exit(2); /* Fatal error */
   }

   /* Cleanup objects created */
   if (AsyncCalls == CCLTRUE) {
      DeleteAnEvent( &Callback[termIndex] );
   }

   if (AllFinished == CCLTRUE) {
      /* Inform main thread all threads are done */

      if (FlagAnEvent( &FinishTest) != CCLTRUE) {
         CCLLOG("Failed to inform main thread via FinishThread - aborting process");
         exit(2); /* Fatal error */
      }
      if (Debug == CCLTRUE) CCLLOG("Flagged main thread");
   }

   if (Debug == CCLTRUE) CCLLOG("Ending thread");
   EndMyThread();
}


/* Function: Response

   Purpose:  Display a return code from an EPI call */

void Response(int Rc, CclThreadId Tid)
{
   char *p;
   char  AsciiRc[32];

   switch (Rc) {
   case CICS_EPI_NORMAL:                p = "CICS_EPI_NORMAL";           break;
   case CICS_EPI_ERR_NOT_INIT:          p = "CICS_EPI_ERR_NOT_INIT";     break;
   case CICS_EPI_ERR_BAD_INDEX:         p = "CICS_EPI_ERR_BAD_INDEX";    break;
   case CICS_EPI_ERR_FAILED:            p = "CICS_EPI_ERR_FAILED";       break;
   case CICS_EPI_ERR_VERSION:           p = "CICS_EPI_ERR_VERSION";      break;
   case CICS_EPI_ERR_IS_INIT:           p = "CICS_EPI_ERR_IS_INIT";      break;
   case CICS_EPI_ERR_NO_SYSTEMS:        p = "CICS_EPI_ERR_NO_SYSTEMS";   break;
   case CICS_EPI_ERR_MORE_SYSTEMS:      p = "CICS_EPI_ERR_MORE_SYSTEMS"; break;
   case CICS_EPI_ERR_MAX_TERMS:         p = "CICS_EPI_ERR_MAX_TERMS";    break;
   case CICS_EPI_ERR_SYSTEM:            p = "CICS_EPI_ERR_SYSTEM";       break;
   case CICS_EPI_ERR_TRAN_ACTIVE:       p = "CICS_EPI_ERR_TRAN_ACTIVE";  break;
   case CICS_EPI_ERR_NO_DATA:           p = "CICS_EPI_ERR_NO_DATA";      break;
   case CICS_EPI_ERR_TTI_ACTIVE:        p = "CICS_EPI_ERR_TTI_ACTIVE";   break;
   case CICS_EPI_ERR_ATI_ACTIVE:        p = "CICS_EPI_ERR_ATI_ACTIVE";   break;
   case CICS_EPI_ERR_NO_CONVERSE:       p = "CICS_EPI_ERR_NO_CONVERSE";  break;
   case CICS_EPI_ERR_ATI_STATE:         p = "CICS_EPI_ERR_ATI_STATE";    break;
   case CICS_EPI_ERR_NO_EVENT:          p = "CICS_EPI_ERR_NO_EVENT";     break;
   case CICS_EPI_ERR_MORE_EVENTS:       p = "CICS_EPI_ERR_MORE_EVENTS";  break;
   case CICS_EPI_ERR_MORE_DATA:         p = "CICS_EPI_ERR_MORE_DATA";    break;
   case CICS_EPI_ERR_WAIT:              p = "CICS_EPI_ERR_WAIT";         break;
   case CICS_EPI_ERR_SERVER_DOWN:       p = "CICS_EPI_ERR_SERVER_DOWN";  break;
   case CICS_EPI_ERR_SENSE_CODE:        p = "CICS_EPI_ERR_SENSE_CODE";   break;
   case CICS_EPI_ERR_SECURITY:          p = "CICS_EPI_ERR_SECURITY";     break;
   case CICS_EPI_ERR_IN_CALLBACK:       p = "CICS_EPI_ERR_IN_CALLBACK";  break;
   case CICS_EPI_ERR_NULL_PARM:         p = "CICS_EPI_ERR_NULL_PARM";    break;
   case CICS_EPI_ERR_ADDTYPE_INVALID:   p = "CICS_EPI_ERR_ADDTYPE_INVALID"; break;
   case CICS_EPI_ERR_SIGNONCAP_INVALID: p = "CICS_EPI_ERR_SIGNONCAP_INVALID"; break;
   case CICS_EPI_ERR_RESPONSE_TIMEOUT:  p = "CICS_EPI_ERR_RESPONSE_TIMEOUT"; break;
   case CICS_EPI_ERR_SIGNON_NOT_POSS:   p = "CICS_EPI_ERR_SIGNON_NOT_POSS"; break;
   case CICS_EPI_ERR_TERMID_INVALID:    p = "CICS_EPI_ERR_TERMID_INVALID"; break;
   case CICS_EPI_ERR_MODELID_INVALID:   p = "CICS_EPI_ERR_MODELID_INVALID"; break;
   case CICS_EPI_ERR_NOT_3270_DEVICE:   p = "CICS_EPI_ERR_NOT_3270_DEVICE"; break;
   case CICS_EPI_ERR_ALREADY_INSTALLED: p = "CICS_EPI_ERR_ALREADY_INSTALLED"; break;
   case CICS_EPI_ERR_USERID_INVALID:    p = "CICS_EPI_ERR_USERID_INVALID"; break;
   case CICS_EPI_ERR_PASSWORD_INVALID:  p = "CICS_EPI_ERR_PASSWORD_INVALID"; break;
   case CICS_EPI_ERR_NULL_USERID:       p = "CICS_EPI_ERR_NULL_USERID"; break;
   case CICS_EPI_ERR_NULL_PASSWORD:     p = "CICS_EPI_ERR_NULL_PASSWORD"; break;
   case CICS_EPI_ERR_CCSID_INVALID:     p = "CICS_EPI_ERR_CCSID_INVALID"; break;
   case CICS_EPI_ERR_SERVER_BUSY:       p = "CICS_EPI_ERR_SERVER_BUSY"; break;
   case CICS_EPI_ERR_ABENDED:           p = "CICS_EPI_ERR_ABENDED"; break;
   case CICS_EPI_ERR_RESOURCE_SHORTAGE: p = "CICS_EPI_ERR_RESOURCE_SHORTAGE"; break;
   case CICS_EPI_ERR_NO_CICS:           p = "CICS_EPI_ERR_NO_CICS"; break;
   case CICS_EPI_ERR_MAX_SYSTEMS:       p = "CICS_EPI_ERR_MAX_SYSTEMS"; break;
   case CICS_EPI_ERR_MAX_SESSIONS:      p = "CICS_EPI_ERR_MAX_SESSIONS"; break;
   default:
      sprintf(AsciiRc, "Unknown (%d)", Rc);
      p = AsciiRc;
      break;
   }
   if (Tid != 0) {
      printf("%d : EPI returned : %s\n", Tid, p);
   } else {
      printf("  EPI returned: %s\n", p);
   }
   fflush(stdout);
}


/* Function: EventType

   Purpose:  Display the event which has arrived */

char* EventType(int event){
   switch (event) {
   case CICS_EPI_EVENT_SEND: return "CICS_EPI_EVENT_SEND";
   case CICS_EPI_EVENT_CONVERSE: return "CICS_EPI_EVENT_CONVERSE";
   case CICS_EPI_EVENT_END_TRAN: return "CICS_EPI_EVENT_END_TRAN";
   case CICS_EPI_EVENT_START_ATI: return "CICS_EPI_EVENT_START_ATI";
   case CICS_EPI_EVENT_END_TERM: return "CICS_EPI_EVENT_END_TERM";
   case CICS_EPI_EVENT_ADD_TERM: return "CICS_EPI_EVENT_ADD_TERM";
   default: return "UNKNOWN_EVENT";
   }
}


/* Function: CallBackProc

   Purpose:  For an ASYNC call, notify the waiting thread that an
             EPI event has arrived */

void CICSEXIT CallBackProc (cics_ushort_t termIndex) {
   if (Debug == CCLTRUE) printf("%d : Callback running for terminal index %d\n",
                                QueryMyThreadId(), termIndex);

   /* Just in case get here too early during an addterminal! */

   while (Callback[termIndex] == 0x00) {
      WaitInMillisecs(1000);
   }

   if (FlagAnEvent( &Callback[termIndex] ) != CCLTRUE) {
      printf("%d : Failed to post callback flag for termidx %d - aborting process\n", QueryMyThreadId(), termIndex);
      fflush(stdout);
      exit(2); /* Fatal error */
   }

   if (Debug == CCLTRUE) {
      printf("%d : Callback completed for terminal index %d\n",
             QueryMyThreadId(), termIndex);
      fflush(stdout);
   }
   return;
}


/* Function: WaitForCICSEvent

   Purpose:  Wait for an EPI event to arrive */

short WaitForCICSEvent(short termIndex, int Idx, int Thread, short *RealRc, CICS_EpiDetails_t *termDetails) {

   short rc;
   CICS_EpiEventData_t EventInfo;

   /* If ASYNC, we do not need to wait on an event as we will be told when one
      arrives through the callback routine, so we could go on and do some
      other processing in the meantime and check the event when we want to.
      For the purposes of this sample, just wait to be told one has arrived */

   if (AsyncCalls == CCLTRUE) {
      if (WaitOnAnEvent( &Callback[termIndex] ) != CCLTRUE) {
         CCLLOG("Failed to wait for call to complete - aborting thread");
         AbortThread();
         EndMyThread();
         return -1;
      }
      if (Debug == CCLTRUE) CCLLOG("Callback was driven!");
      if (ResetAnEvent( &Callback[termIndex] ) != CCLTRUE) {
         CCLLOG("Failed to reset for callback event - aborting thread");
         AbortThread();
         EndMyThread();
         return -1;
      }
   }


   /* Do an EPI GetEvent call to retrieve the event which has arrived (and in
      SYNC mode wait for it to arrive as well) */

   EventInfo.Data = malloc(BUFFER_SIZE);
   EventInfo.Size = BUFFER_SIZE;

   if (Debug == CCLTRUE) CCLLOG("Waiting for an event");
   rc = CICS_EpiGetEvent(termIndex, CICS_EPI_WAIT, &EventInfo);
   if (Debug == CCLTRUE) CCLLOG("Event has arrived");

   if (rc != CICS_EPI_NORMAL && rc != CICS_EPI_ERR_MORE_EVENTS) {
      CCLLOG("CICS_EpiGetEvent error - aborting thread");
      Response(rc, QueryMyThreadId());
      AbortThread();
      EndMyThread();
   }

   *RealRc = EventInfo.EndReturnCode;

   if (termDetails != NULL && EventInfo.Event == CICS_EPI_EVENT_ADD_TERM) {
      memcpy(termDetails, EventInfo.Data, sizeof(CICS_EpiDetails_t));
   }

   if (Debug) {
      CCLLOG2("Event received for term %d, event %s, %s",
              termIndex, EventType(EventInfo.Event),
              "EndReturnCode follows");

      Response(EventInfo.EndReturnCode, QueryMyThreadId());

      if (EventInfo.Event == CICS_EPI_EVENT_SEND ||
          EventInfo.Event == CICS_EPI_EVENT_CONVERSE) {
         CCLLOG("SEND/CONVERSE received with data:");
         DumpDataStream((cics_char_t *)EventInfo.Data, EventInfo.Size);
      }

      if (EventInfo.Event == CICS_EPI_EVENT_END_TRAN ||
          EventInfo.Event == CICS_EPI_EVENT_END_TERM) {
         CCLLOG2("%s EndReason is %s%s",
                 EventType(EventInfo.Event),
                 EndReason(EventInfo.EndReason),"");
      }

   }

   if ((rc == CICS_EPI_ERR_MORE_EVENTS) && (AsyncCalls == CCLTRUE)) {
      if (Debug) CCLLOG("More events, so reflag callback event");
      FlagAnEvent(&Callback[termIndex] );
   }

   free(EventInfo.Data);
   return EventInfo.Event;
}

char AsciiReason[32];


/* Display the EPI EndReason */

char *EndReason(CICS_EpiEnd_t endReason) {
   switch (endReason) {
   case CICS_EPI_TRAN_NO_ERROR: return "CICS_EPI_TRAN_NO_ERROR";
   case CICS_EPI_TRAN_NOT_STARTED: return "CICS_EPI_TRAN_NOT_STARTED";
   case CICS_EPI_TRAN_STATE_UNKNOWN: return "CICS_EPI_TRAN_STATE_UNKNOWN";
   case CICS_EPI_READTIMEOUT_EXPIRED: return "CICS_EPI_READTIMEOUT_EXPIRED";
   case CICS_EPI_END_SIGNOFF        : return "CICS_EPI_END_SIGNOFF";
   case CICS_EPI_END_SHUTDOWN       : return "CICS_EPI_END_SHUTDOWN";
   case CICS_EPI_END_OUTSERVICE     : return "CICS_EPI_END_OUTSERVICE";
   case CICS_EPI_END_UNKNOWN        : return "CICS_EPI_END_UNKNOWN";
   case CICS_EPI_END_FAILED         : return "CICS_EPI_END_FAILED";
   default:
      break;
   }
   sprintf(AsciiReason, "Unknown reason (%d)", endReason);
   return AsciiReason;
}


/* Display the inbound data stream */

void DumpDataStream(cics_char_t *Start, cics_ushort_t Size) {
   int row, column;

   for (row = 0; row*16 < Size; row++) {
      printf("     ");
      for (column = 0; column < 16; column++) {
         if (row*16 + column < Size)
            printf("%2.2X", (unsigned char)Start[row*16 + column]);
         else
            printf("  ");
      }
      printf("   ");
      for (column = 0; row*16 + column < Size && column < 16; column++) {
         if (isprint(Start[row*16 + column]))
            printf("%c", Start[row*16 + column]);
         else
            printf(".");
      }
      printf("\n");
   }
}

void AbortThread(void) {
   CclBool AllFinished = CCLFALSE;

   if (GetExclusiveAccess( &ProgressLock ) != CCLTRUE) {
      printf("%d : Failed to get exclusive access to progress structure on thread abort - aborting process\n", QueryMyThreadId());
      fflush(stdout);
      exit(2); /* Terminate process, fatal error */
   }

   Progress.NumCompleteError++;

   /* Check the last thread did not abort */

   if (Progress.NumCompleteOK + Progress.NumCompleteError == Progress.TotalThreads) {
      AllFinished = CCLTRUE;
   }

   if (ReleaseExclusiveAccess( &ProgressLock ) != CCLTRUE) {
      printf("%d : Failed to release exclusive access to progress structure on thread abort - aborting process\n", QueryMyThreadId());
      fflush(stdout);
      exit(2); /* Terminate process, fatal error */
   }

   if (AllFinished == CCLTRUE) {
      /* Inform main thread all threads are done */

      if (FlagAnEvent( &FinishTest) != CCLTRUE) {
         printf("%d : Failed to inform main thread of test completion - aborting process\n", QueryMyThreadId());
         fflush(stdout);
         exit(2); /* Terminate process, fatal error */
      }
   }

}


/* Function: StrToupper

   Purpose:  Takes an array of characters and convert them to upper case */

char *StrToupper(char *str) {
        int i = 0;

        for(i = 0; i < strlen(str); i++)
                if (str[i] >= 'a' && str[i] <= 'z') str[i] = str[i] - 32;

        return str;
}
