/* File Name(s)  : epib1.c

   Product       : CICS Transaction Gateway

   Description   : This sample shows how to connect to a CICS server using the
                   EPI API. It starts a transaction on a chosen server and
                   outputs the reply to the console.

   Prerequisites:  This Sample requires the supplied EP01 transaction to be
                   installed on your CICS server.

 Licensed Materials - Property of IBM  
  
 5724-I81,5725-B65 
  
 (C) Copyright IBM Corp. 1999, 2014 All Rights Reserved.  
  
 US Government Users Restricted Rights - Use, duplication or  
 disclosure restricted by GSA ADP Schedule Contract with  
 IBM Corp.  
   
   Status: Version 9 Release 2 

                   The following code is sample code created by IBM
                   Corporation.  This sample code is not part of any standard
                   IBM product and is provided to you solely for the purpose
                   of assisting you in the development of your applications.
                   The code is provided 'AS IS', without warranty or
                   condition of any kind.  IBM shall not be liable for any
                   damages arising out of your use of the sample code, even
                   if IBM has been advised of the possibility of such damages. */


/* Include files */

#include <stdio.h>
#include <string.h>
#include <stddef.h>
#include <stdlib.h>
#include <ctype.h>

#include <cics_epi.h>
#include <dfhaid.h>


/* Structures, defines and macro definitions */

#define BUFFER_SIZE 1000
#define CCLFALSE    0
#define CCLTRUE     1

#define copyright_notice copyright4cepib1
char version4cepib1[] =
"\nCICS Transaction Gateway"
" Version 9.2.0.3"
" Build Level c920-20180312-1735" ;

char copyright_notice[] =
"\n"
"Licensed Materials - Property of IBM. "
"\n"
"5724-I81,5725-B65 "
"\n"
"(c) Copyright IBM Corp. 1999, 2014 All Rights Reserved. "
"\n"
"US Government Users Restricted Rights - Use, duplication or "
"disclosure restricted by GSA ADP Schedule Contract with "
"IBM Corp."
"\n";

typedef struct {
   char               AIDKey;
   char               Byte0;
   char               Byte1;
   char               TransactionCode[4];
   char               Data[BUFFER_SIZE];
} DataStream;


/* Function prototypes */

void  Response(int Rc);
char *EventType(int event);
void  DumpDataStream(cics_char_t *Start, cics_ushort_t Size);
char *EndReason(CICS_EpiEnd_t endReason);


/* Main Function */

int main(int argc, char *argv[]){

   CICS_EpiSystem_t servers[20];             /*CICS_EciListSystems Server list */
   cics_ushort_t    serverCount = 20;        /*Length counter for above list   */
   cics_char_t      chosenServer[CICS_EPI_SYSTEM_MAX +1];/*Selected Server name*/
   cics_char_t      transid[CICS_EPI_TRANSID_MAX + 1] = "EP01"; /*Transaction to execute.*/

   CICS_EpiDetails_t   termDetails;
   CICS_EpiEventData_t eventData;
   void               *dataPointer;
   cics_ushort_t       termIndex;
   DataStream          epiData;

   int  rc, i;
   int  loop   = 0;
   int  choice = 0;
   int  Debug  = CCLFALSE;
   char tempstr[255];


   /* Display Banner and syntax of program */

   printf("CICS Transaction Gateway Basic EPI Sample\n\n");

   if (argc > 1) {
      Debug = CCLTRUE;
      printf("  Running with debug enabled\n");
   }


   /* Initialise the EPI */

   printf("\nInitialising EPI\n");
   rc = CICS_EpiInitialize(CICS_EPI_VERSION_200);


   /* Ask the Client daemon for a list of servers to connect to */

   rc = CICS_EpiListSystems(NULL, &serverCount, servers);
   if (rc == CICS_EPI_NORMAL) {
      printf("\nServers defined in the Client daemon:\n");

      do {

         /* Display the list and get users input as to which one to use */

         for (i=0; i<serverCount;i++) {
            printf("   %d.%s\n",i+1 , servers[i].SystemName);
         }

         printf("\nChoose server to connect to:\n");
         scanf("%s", tempstr);
         choice = atol(tempstr);


         /* Validate choice and if ok exit the loop */

         if (choice > 0 && choice < serverCount+1) {
            strcpy(chosenServer,servers[choice-1].SystemName);
            loop=0;
         } else {
            printf("Out of range\n");
            loop=1;
         }
      }while (loop==1);


      /* Create a Terminal on the chosen server */

      printf("Creating a terminal on server %s\n", chosenServer);
      rc = CICS_EpiAddTerminal(NULL, chosenServer, NULL, NULL, NULL,
                               &termDetails, &termIndex);
      if (rc == CICS_EPI_NORMAL) {


         /* Output some information about the terminal */

         printf("  Connected on terminal: %s\n", termDetails.TermId);
         printf("  Terminal dimensions = %dx%d\n", termDetails.NumLines,
                                                   termDetails.NumColumns);

         /* Set up the epi 3270 data stream and start transaction on server */

         epiData.AIDKey = 0x27;
         epiData.Byte0  = 0x20;
         epiData.Byte1  = 0x20;
         memcpy (epiData.TransactionCode, transid,4);
         memset (epiData.Data, 0, BUFFER_SIZE);

         printf("\n  Starting %s on %s\n", transid, chosenServer);
         dataPointer = &epiData.AIDKey;
         rc = CICS_EpiStartTran(termIndex, NULL,
                                dataPointer, offsetof(DataStream, Data));
         if (rc == CICS_EPI_NORMAL) {

            /* Get some events until we get an END_TRAN event */

            do {
               loop = 1;
               eventData.Size = sizeof(DataStream);
               eventData.Data = dataPointer;

               rc = CICS_EpiGetEvent(termIndex, CICS_EPI_WAIT, &eventData);
               if (rc != CICS_EPI_NORMAL && rc != CICS_EPI_ERR_MORE_EVENTS) {

                  /* Handle errors */

                  Response(rc);
                  loop = 0;
               } else if (eventData.Event == CICS_EPI_EVENT_END_TRAN) {

                  /* Transaction has terminated */

                  printf("  End of transaction; end reason %s\n",
                         EndReason(eventData.EndReason));
                  if (eventData.EndReason != CICS_EPI_TRAN_NO_ERROR) {
                     Response(eventData.EndReturnCode);
                     loop = 0;
                  } else {
                     strcpy(transid, (char *)&eventData.TransId);
                     loop = 0;
                  }
               } else {

                  /* Data received from CICS (should confirm EPI_EVENT) */

                  printf("  Event type %s received from CICS\n",
                         EventType(eventData.Event));
                  if (Debug == CCLTRUE) {
                    printf("  Data stream returned:\n");
                    DumpDataStream((cics_char_t *)eventData.Data, eventData.Size);
                  } else {
                    printf("  Data returned: %s\n",
                                   (cics_char_t *)(eventData.Data+2));
                  }
               }
            } while (loop);

         } else {
            Response(rc);
            return 1;
         }


         /* Delete the terminal */

         rc = CICS_EpiDelTerminal(termIndex);
         if (rc == CICS_EPI_NORMAL) {


            /* Get some events until we get an END_TRAN event */

            do {
               loop = 1;
               eventData.Size = sizeof(DataStream);
               eventData.Data = dataPointer;

               rc = CICS_EpiGetEvent(termIndex, CICS_EPI_WAIT, &eventData);
               if (rc != CICS_EPI_NORMAL && rc != CICS_EPI_ERR_MORE_EVENTS) {

                  /* Handle errors */

                  Response(rc);
                  loop = 0;
               } else if (eventData.Event == CICS_EPI_EVENT_END_TERM) {

                  /* Terminal has been deleted */

                  printf("Terminal deleted; end reason %s\n",
                            EndReason(eventData.EndReason));
                  if (eventData.EndReason != CICS_EPI_TRAN_NO_ERROR) {
                     Response(eventData.EndReturnCode);
                  }
                  loop = 0;
               } else {

                  /* Data received from CICS (should confirm EPI_EVENT) */

                  printf("  Event type %s received from CICS\n",
                             EventType(eventData.Event));
                  printf("  Data stream returned:\n");
                  DumpDataStream((cics_char_t *)eventData.Data, eventData.Size);
               }
            } while (loop);

         } else {
            Response(rc);
            return 1;
         }

      } else {
         Response(rc);
         return 1;
      }
   } else {
      Response(rc);
      return 1;
   }


   /* Shut down the EPI */

   printf("\nTerminating EPI\n");
   rc = CICS_EpiTerminate();

   fflush(stdout);
   return 0;
}


/* Display the EPI return code */

void Response(int Rc){
   char             *p;
   char              AsciiRc[32];

   switch (Rc) {
   case CICS_EPI_NORMAL:                p = "CICS_EPI_NORMAL";           break;
   case CICS_EPI_ERR_NOT_INIT:          p = "CICS_EPI_ERR_NOT_INIT";     break;
   case CICS_EPI_ERR_BAD_INDEX:         p = "CICS_EPI_ERR_BAD_INDEX";    break;
   case CICS_EPI_ERR_FAILED:            p = "CICS_EPI_ERR_FAILED";       break;
   case CICS_EPI_ERR_VERSION:           p = "CICS_EPI_ERR_VERSION";      break;
   case CICS_EPI_ERR_IS_INIT:           p = "CICS_EPI_ERR_IS_INIT";      break;
   case CICS_EPI_ERR_NO_SYSTEMS:        p = "CICS_EPI_ERR_NO_SYSTEMS";   break;
   case CICS_EPI_ERR_MORE_SYSTEMS:      p = "CICS_EPI_ERR_MORE_SYSTEMS"; break;
   case CICS_EPI_ERR_MAX_TERMS:         p = "CICS_EPI_ERR_MAX_TERMS";    break;
   case CICS_EPI_ERR_SYSTEM:            p = "CICS_EPI_ERR_SYSTEM";       break;
   case CICS_EPI_ERR_TRAN_ACTIVE:       p = "CICS_EPI_ERR_TRAN_ACTIVE";  break;
   case CICS_EPI_ERR_NO_DATA:           p = "CICS_EPI_ERR_NO_DATA";      break;
   case CICS_EPI_ERR_TTI_ACTIVE:        p = "CICS_EPI_ERR_TTI_ACTIVE";   break;
   case CICS_EPI_ERR_ATI_ACTIVE:        p = "CICS_EPI_ERR_ATI_ACTIVE";   break;
   case CICS_EPI_ERR_NO_CONVERSE:       p = "CICS_EPI_ERR_NO_CONVERSE";  break;
   case CICS_EPI_ERR_ATI_STATE:         p = "CICS_EPI_ERR_ATI_STATE";    break;
   case CICS_EPI_ERR_NO_EVENT:          p = "CICS_EPI_ERR_NO_EVENT";     break;
   case CICS_EPI_ERR_MORE_EVENTS:       p = "CICS_EPI_ERR_MORE_EVENTS";  break;
   case CICS_EPI_ERR_MORE_DATA:         p = "CICS_EPI_ERR_MORE_DATA";    break;
   case CICS_EPI_ERR_WAIT:              p = "CICS_EPI_ERR_WAIT";         break;
   case CICS_EPI_ERR_SERVER_DOWN:       p = "CICS_EPI_ERR_SERVER_DOWN";  break;
   case CICS_EPI_ERR_SENSE_CODE:        p = "CICS_EPI_ERR_SENSE_CODE";   break;
   case CICS_EPI_ERR_SECURITY:          p = "CICS_EPI_ERR_SECURITY";     break;
   case CICS_EPI_ERR_IN_CALLBACK:       p = "CICS_EPI_ERR_IN_CALLBACK";  break;
   case CICS_EPI_ERR_NULL_PARM:         p = "CICS_EPI_ERR_NULL_PARM";    break;
   case CICS_EPI_ERR_ADDTYPE_INVALID:   p = "CICS_EPI_ERR_ADDTYPE_INVALID"; break;
   case CICS_EPI_ERR_SIGNONCAP_INVALID: p = "CICS_EPI_ERR_SIGNONCAP_INVALID"; break;
   case CICS_EPI_ERR_RESPONSE_TIMEOUT:  p = "CICS_EPI_ERR_RESPONSE_TIMEOUT"; break;
   case CICS_EPI_ERR_SIGNON_NOT_POSS:   p = "CICS_EPI_ERR_SIGNON_NOT_POSS"; break;
   case CICS_EPI_ERR_TERMID_INVALID:    p = "CICS_EPI_ERR_TERMID_INVALID"; break;
   case CICS_EPI_ERR_MODELID_INVALID:   p = "CICS_EPI_ERR_MODELID_INVALID"; break;
   case CICS_EPI_ERR_NOT_3270_DEVICE:   p = "CICS_EPI_ERR_NOT_3270_DEVICE"; break;
   case CICS_EPI_ERR_ALREADY_INSTALLED: p = "CICS_EPI_ERR_ALREADY_INSTALLED"; break;
   case CICS_EPI_ERR_USERID_INVALID:    p = "CICS_EPI_ERR_USERID_INVALID"; break;
   case CICS_EPI_ERR_PASSWORD_INVALID:  p = "CICS_EPI_ERR_PASSWORD_INVALID"; break;
   case CICS_EPI_ERR_NULL_USERID:       p = "CICS_EPI_ERR_NULL_USERID";  break;
   case CICS_EPI_ERR_NULL_PASSWORD:     p = "CICS_EPI_ERR_NULL_PASSWORD"; break;
   case CICS_EPI_ERR_CCSID_INVALID:     p = "CICS_EPI_ERR_CCSID_INVALID"; break;
   case CICS_EPI_ERR_SERVER_BUSY:       p = "CICS_EPI_ERR_SERVER_BUSY";  break;
   case CICS_EPI_ERR_ABENDED:           p = "CICS_EPI_ERR_ABENDED";      break;
   case CICS_EPI_ERR_RESOURCE_SHORTAGE: p = "CICS_EPI_ERR_RESOURCE_SHORTAGE"; break;
   case CICS_EPI_ERR_NO_CICS:           p = "CICS_EPI_ERR_NO_CICS";      break;
   case CICS_EPI_ERR_MAX_SYSTEMS:       p = "CICS_EPI_ERR_MAX_SYSTEMS";  break;
   case CICS_EPI_ERR_MAX_SESSIONS:      p = "CICS_EPI_ERR_MAX_SESSIONS"; break;
   default:
      sprintf(AsciiRc, "Unknown (%d)", Rc);
      p = AsciiRc;
      break;
   }
   printf("  EPI returned: %s\n", p);
   fflush(stdout);
}


/* Display the EPI event type */

char* EventType(int event){
   switch (event) {
   case 1: return "CICS_EPI_EVENT_SEND";
   case 2: return "CICS_EPI_EVENT_CONVERSE";
   case 3: return "CICS_EPI_EVENT_END_TRAN";
   case 4: return "CICS_EPI_EVENT_START_ATI";
   case 5: return "CICS_EPI_EVENT_END_TERM";
   default: return "UNKNOWN_EVENT";
   }
}

char AsciiReason[32];


/* Display the EPI end reason */

char *EndReason(CICS_EpiEnd_t endReason){
   switch (endReason) {
   case CICS_EPI_TRAN_NO_ERROR: return "CICS_EPI_TRAN_NO_ERROR";
   case CICS_EPI_TRAN_NOT_STARTED: return "CICS_EPI_TRAN_NOT_STARTED";
   case CICS_EPI_TRAN_STATE_UNKNOWN: return "CICS_EPI_TRAN_STATE_UNKNOWN";
   case CICS_EPI_READTIMEOUT_EXPIRED: return "CICS_EPI_READTIMEOUT_EXPIRED";
   case CICS_EPI_END_SIGNOFF        : return "CICS_EPI_END_SIGNOFF";
   case CICS_EPI_END_SHUTDOWN       : return "CICS_EPI_END_SHUTDOWN";
   case CICS_EPI_END_OUTSERVICE     : return "CICS_EPI_END_OUTSERVICE";
   case CICS_EPI_END_UNKNOWN        : return "CICS_EPI_END_UNKNOWN";
   case CICS_EPI_END_FAILED         : return "CICS_EPI_END_FAILED";
   default:
      break;
   }
   sprintf(AsciiReason, "Unknown reason (%d)", endReason);
   return AsciiReason;
}


/* Display the inbound data stream */

void DumpDataStream(cics_char_t *Start, cics_ushort_t Size){
   int row, column;

   for (row = 0; row*16 < Size; row++) {
      printf("     ");
      for (column = 0; column < 16; column++) {
         if (row*16 + column < Size){
            printf("%2.2X", (unsigned char)Start[row*16 + column]);
         } else {
            printf("  ");
         }
      }
      printf("   ");
      for (column = 0; row*16 + column < Size && column < 16; column++) {
         if (isprint(Start[row*16 + column])){
            printf("%c", Start[row*16 + column]);
         } else {
            printf(".");
         }
      }
      printf("\n");
   }
}
