/* File Name(s)  : epii1.c

   Product       : CICS Transaction Gateway

   Description   : This sample shows the basic use of the CICS TG Client API.
                   It queries the Client daemon for a list of available servers 
                   and then launches a transaction on the server chosen.

   Prerequisites:  This samples requires the CICS Transaction EP01 to be
                   installed on your CICS server for operation.

 Licensed Materials - Property of IBM  
  
 5724-I81,5725-B65 
  
 (C) Copyright IBM Corp. 1999, 2014 All Rights Reserved.  
  
 US Government Users Restricted Rights - Use, duplication or  
 disclosure restricted by GSA ADP Schedule Contract with  
 IBM Corp.  
   
   Status: Version 9 Release 2 

                   The following code is sample code created by IBM
                   Corporation.  This sample code is not part of any standard
                   IBM product and is provided to you solely for the purpose
                   of assisting you in the development of your applications.
                   The code is provided 'AS IS', without warranty or
                   condition of any kind. IBM shall not be liable for any
                   damages arising out of your use of the sample code, even
                   if IBM has been advised of the possibility of such damages. */


/* Include files */

#include <stdio.h>
#include <string.h>
#include <stddef.h>
#include <stdlib.h>
#include <ctype.h>

#include <cics_epi.h>
#include <cics3270.h>
#include <cclcalls.h>


/* Structures, defines and macro definitions */

#define DATASTREAM_LENGTH 4096

#define copyright_notice copyright4cepii1
char version4cepii1[] =
"\nCICS Transaction Gateway"
" Version 9.2.0.3"
" Build Level c920-20180312-1735" ;

char copyright_notice[] =
"\n"
"Licensed Materials - Property of IBM. "
"\n"
"5724-I81,5725-B65 "
"\n"
"(c) Copyright IBM Corp. 1999, 2014 All Rights Reserved. "
"\n"
"US Government Users Restricted Rights - Use, duplication or "
"disclosure restricted by GSA ADP Schedule Contract with "
"IBM Corp."
"\n";

typedef struct
{
    cics_ubyte_t AIDKey;
    cics_ubyte_t Byte0;
    cics_ubyte_t Byte1;
    cics_ubyte_t Data[DATASTREAM_LENGTH];
} EpiDataStream_t;


/* Global Variables */

CclEvent Callback; /* event for callback to signal that EPI event has arrived */
char AsciiEvent[32];                   /* buffer for 'unknown event' message  */
char AsciiReason[32];                  /* buffer for 'unknown reason' message */
int  Debug = CCLFALSE;                 /* Is debugging the datastreams enabled*/


/* Function prototypes */

void Response(cics_sshort_t Rc);
char *EventType(CICS_EpiEvent_t event);
void displayDetails (CICS_EpiDetails_t *det);
char *EndReason(CICS_EpiEnd_t endReason);
void CICSEXIT CallBackProc (cics_ushort_t termIndex);
void DumpDataStream(cics_char_t *Start, cics_ushort_t Size);
cics_sshort_t GetScreen(cics_ushort_t termIndex, cics_char_t *TransId,
                        cics_ubyte_t *DataPtr, cics_ushort_t *DataSize);


/* Main Function */

int main(int argc, char *argv[])
{
    CICS_EpiSystem_t     servers[20];
    cics_ushort_t        serverCount = 20;
    cics_char_t          chosenServer[CICS_EPI_SYSTEM_MAX +1];
    cics_char_t          nextTransId[4 + 1];
    CICS_EpiDetails_t    myDetails;
    CICS_EpiAttributes_t myAttr;
    CICS_EpiEventData_t  myEventData;
    EpiDataStream_t      dataStream;
    cics_ubyte_t        *dataPtr;
    cics_ushort_t        streamSize;
    cics_ushort_t        termIndex;
    int                  i;
    int                  choice = 0;
    cics_sshort_t        rc;
    char                 tempstr[255];
    int                  loop;


    /* Initialise the EPI with latest version of the API */

    rc = CICS_EpiInitialize(CICS_EPI_VERSION_200);
    if (rc != CICS_EPI_NORMAL)
    {
        Response(rc);
        return 1;
    }


    /* Display banner */

    printf("CICS Transaction Gateway Intermediate EPI Sample\n\n");

    if (argc > 1) {
       Debug = CCLTRUE;
       printf("  Running with debug enabled\n");
    }

    printf("\nServers defined in the Client daemon:\n");


    /* Ask the Client daemon for a list of servers to connect to */

    rc = CICS_EpiListSystems(NULL, &serverCount, servers);
    if (rc == CICS_EPI_NORMAL)
    {

        /* Display server list to user */

        for (i=0; i < serverCount; i++)
        {
            printf("   %d.%s\n", i + 1, servers[i].SystemName);
        }
    }
    else
    {
        Response(rc);
        return 1;
    }


    loop = CCLTRUE;
    do
    {

        /* Ask user to chose the server to call */

        printf("\nChoose server to connect to:\n");
        scanf("%s", tempstr);
        choice = (cics_ushort_t) atol(tempstr);


        /* Check if choice is within range */

        if (choice > 0 && choice < serverCount+1)
        {
            strcpy(chosenServer, servers[choice-1].SystemName);
            loop = CCLFALSE;
            break;
        }
        else
        {
            printf("%d is out of range\n", choice);
        }
    } while (loop == CCLTRUE);

    if(CreateAnEvent(NULL, &Callback) != CCLTRUE)
    {
        printf("Failed to create callback event - aborting\n");
        fflush(stdout);
        return 1;
    }


    /* Set the attributes for an extended terminal */

    memset(&myDetails, '\0', sizeof(CICS_EpiDetails_t));
    memset(&myAttr, '\0', sizeof(CICS_EpiAttributes_t));
    myAttr.EpiAddType = CICS_EPI_ADD_ASYNC;
    myAttr.SignonCapability = CICS_EPI_SIGNON_INCAPABLE;
    myAttr.InstallTimeOut = 30;


    /* Install an extended terminal */
    /* Cast needed for HP-UX        */
    rc = CICS_EpiAddExTerminal(chosenServer, NULL, NULL,
                               (CICS_EpiNotify_t)&CallBackProc,
                               &myDetails, &termIndex, &myAttr);
    if(rc != CICS_EPI_NORMAL)
    {
        Response(rc);
        return 1;
    }

    printf("Waiting for the terminal to install\n");
    if(WaitOnAnEvent( &Callback ) != CCLTRUE)
    {


        /* Wait for Callback to flag an event */

        printf("Failed to wait for event - aborting\n");
        fflush(stdout);
        return 1;
    }

    myEventData.Data = (cics_ubyte_t *)&myDetails;
    myEventData.Size = sizeof(myDetails);
    rc = CICS_EpiGetEvent(termIndex, CICS_EPI_NOWAIT, &myEventData);
    if(rc != CICS_EPI_NORMAL)
    {
        Response(rc);
        return 1;
    }
    if(myEventData.Event != CICS_EPI_EVENT_ADD_TERM
       || myEventData.EndReturnCode != CICS_EPI_NORMAL)
    {
        Response(myEventData.EndReturnCode);
        return 1;
    }

    displayDetails(&myDetails);

    /*
       Initialise the Data section to 0's
       This means it will always be null terminated if we
       want to view it as a string later on.
     */

    memset(dataStream.Data,0,DATASTREAM_LENGTH);


    /* Set up the 3270 datastream data structure */

    dataStream.AIDKey = CICS_EPI_AID_ENTER;


    /* position cursor at top left corner of screen */

    dataStream.Byte0 = 0x20;
    dataStream.Byte1 = 0x20;
    memcpy(dataStream.Data, "EP01", 4);
    dataPtr = (cics_ubyte_t *)&dataStream.AIDKey;

    printf("Start transaction\n");
    rc = CICS_EpiStartTran(termIndex, NULL, dataPtr,
                           offsetof(EpiDataStream_t,Data) + 4);
    if(rc != CICS_EPI_NORMAL)
    {
        Response(rc);
        return 1;
    }


    /* A 3270 datastream consists of a one byte command code, a one byte
       Write Control Character, and then variable data. We do not want to
       send the first two bytes back; so we have them written to Byte0
       and Byte1. */

    streamSize = DATASTREAM_LENGTH + 2;
    rc = GetScreen(termIndex, nextTransId,
                   (cics_ubyte_t *)&dataStream.Byte0, &streamSize);
    if(rc != CICS_EPI_NORMAL)
    {
        Response(rc);
        return 1;
    }


    /* Set up the 3270 datastream data structure */

    dataStream.AIDKey = CICS_EPI_AID_ENTER;
    dataStream.Byte0 = 0x20;
    dataStream.Byte1 = 0x20;
    dataPtr = (cics_ubyte_t *)&(dataStream.AIDKey);

    printf("Start next part of pseudo-conversation\n");
    rc = CICS_EpiStartTran(termIndex, nextTransId, (cics_ubyte_t *)&dataStream,
                           (cics_ushort_t)(offsetof(EpiDataStream_t, Byte0)
                                           + streamSize));
    if(rc != CICS_EPI_NORMAL)
    {
        Response(rc);
        return 1;
    }

    streamSize = DATASTREAM_LENGTH + 2;
    rc = GetScreen(termIndex, nextTransId,
                   (cics_ubyte_t *)&dataStream.Byte0, &streamSize);
    if(rc != CICS_EPI_NORMAL)
    {
        Response(rc);
        return 1;
    }


    /* Finally remove the terminal. CICS_EpiDelTerminal could be used instead */

    rc = CICS_EpiPurgeTerminal(termIndex);
    if(rc != CICS_EPI_NORMAL)
    {
        Response(rc);
        return 1;
    }
    printf("Waiting for the terminal to be deleted\n");
    if(WaitOnAnEvent( &Callback ) != CCLTRUE)
    {

        /* Wait for Callback to flag an event */

        printf("Failed to wait for event - aborting\n");
        fflush(stdout);
        return 1;
    }
    myEventData.Data = (cics_ubyte_t *)&myDetails;
    myEventData.Size = sizeof(myDetails);
    rc = CICS_EpiGetEvent(termIndex, CICS_EPI_NOWAIT, &myEventData);
    if(rc != CICS_EPI_NORMAL)
    {
        Response(rc);
        return 1;
    }
    if(myEventData.Event != CICS_EPI_EVENT_END_TERM
       || myEventData.EndReturnCode != CICS_EPI_NORMAL)
    {
        Response(myEventData.EndReturnCode);
        return 1;
    }
    fflush(stdout);
    return 0;
}


/* Function: displayDetails

   Purpose:  Display the details of the installed terminal resource */

void displayDetails (CICS_EpiDetails_t *det)
{
    printf("Terminal installed with following characteristics\n");
    printf("\tNetName = %s\n", (det->NetName ? det->NetName : "NULL") );
    printf("\tNumLines = %d\n", det->NumLines);
    printf("\tNumColumns = %d\n", det->NumColumns);
    printf("\tMaxData = %d\n", det->MaxData);
    printf("\tErrLastLine = %d\n", det->ErrLastLine);
    printf("\tErrIntensify = %d\n", det->ErrIntensify);
    printf("\tErrColor = %d\n", det->ErrColor);
    printf("\tErrHilight = %d\n", det->ErrHilight);
    printf("\tColor = %d\n", det->Color);
    printf("\tSystem = %s\n", det->System);
    printf("\tTermId = %s\n", det->TermId);
    printf("\tSignonCapability = %d\n", det->SignonCapability);
    printf("\n");
}

/* Function: Response

   Purpose:  Display the immediate response code from an EPI call */

void Response(cics_sshort_t Rc)
{
    char *p;
    char  AsciiRc[32];

    switch(Rc)
    {
    case CICS_EPI_NORMAL               : p = "CICS_EPI_NORMAL"               ; break;
    case CICS_EPI_ERR_NOT_INIT         : p = "CICS_EPI_ERR_NOT_INIT"         ; break;
    case CICS_EPI_ERR_BAD_INDEX        : p = "CICS_EPI_ERR_BAD_INDEX"        ; break;
    case CICS_EPI_ERR_FAILED           : p = "CICS_EPI_ERR_FAILED"           ; break;
    case CICS_EPI_ERR_VERSION          : p = "CICS_EPI_ERR_VERSION"          ; break;
    case CICS_EPI_ERR_IS_INIT          : p = "CICS_EPI_ERR_IS_INIT"          ; break;
    case CICS_EPI_ERR_NO_SYSTEMS       : p = "CICS_EPI_ERR_NO_SYSTEMS"       ; break;
    case CICS_EPI_ERR_MORE_SYSTEMS     : p = "CICS_EPI_ERR_MORE_SYSTEMS"     ; break;
    case CICS_EPI_ERR_MAX_TERMS        : p = "CICS_EPI_ERR_MAX_TERMS"        ; break;
    case CICS_EPI_ERR_SYSTEM           : p = "CICS_EPI_ERR_SYSTEM"           ; break;
    case CICS_EPI_ERR_TRAN_ACTIVE      : p = "CICS_EPI_ERR_TRAN_ACTIVE"      ; break;
    case CICS_EPI_ERR_NO_DATA          : p = "CICS_EPI_ERR_NO_DATA"          ; break;
    case CICS_EPI_ERR_TTI_ACTIVE       : p = "CICS_EPI_ERR_TTI_ACTIVE"       ; break;
    case CICS_EPI_ERR_ATI_ACTIVE       : p = "CICS_EPI_ERR_ATI_ACTIVE"       ; break;
    case CICS_EPI_ERR_NO_CONVERSE      : p = "CICS_EPI_ERR_NO_CONVERSE"      ; break;
    case CICS_EPI_ERR_ATI_STATE        : p = "CICS_EPI_ERR_ATI_STATE"        ; break;
    case CICS_EPI_ERR_NO_EVENT         : p = "CICS_EPI_ERR_NO_EVENT"         ; break;
    case CICS_EPI_ERR_MORE_EVENTS      : p = "CICS_EPI_ERR_MORE_EVENTS"      ; break;
    case CICS_EPI_ERR_MORE_DATA        : p = "CICS_EPI_ERR_MORE_DATA"        ; break;
    case CICS_EPI_ERR_WAIT             : p = "CICS_EPI_ERR_WAIT"             ; break;
    case CICS_EPI_ERR_SERVER_DOWN      : p = "CICS_EPI_ERR_SERVER_DOWN"      ; break;
    case CICS_EPI_ERR_SENSE_CODE       : p = "CICS_EPI_ERR_SENSE_CODE"       ; break;
    case CICS_EPI_ERR_SECURITY         : p = "CICS_EPI_ERR_SECURITY"         ; break;
    case CICS_EPI_ERR_IN_CALLBACK      : p = "CICS_EPI_ERR_IN_CALLBACK"      ; break;
    case CICS_EPI_ERR_NULL_PARM        : p = "CICS_EPI_ERR_NULL_PARM"        ; break;
    case CICS_EPI_ERR_ADDTYPE_INVALID  : p = "CICS_EPI_ERR_ADDTYPE_INVALID"  ; break;
    case CICS_EPI_ERR_SIGNONCAP_INVALID: p = "CICS_EPI_ERR_SIGNONCAP_INVALID"; break;
    case CICS_EPI_ERR_RESPONSE_TIMEOUT : p = "CICS_EPI_ERR_RESPONSE_TIMEOUT" ; break;
    case CICS_EPI_ERR_SIGNON_NOT_POSS  : p = "CICS_EPI_ERR_SIGNON_NOT_POSS"  ; break;
    case CICS_EPI_ERR_TERMID_INVALID   : p = "CICS_EPI_ERR_TERMID_INVALID"   ; break;
    case CICS_EPI_ERR_MODELID_INVALID  : p = "CICS_EPI_ERR_MODELID_INVALID"  ; break;
    case CICS_EPI_ERR_NOT_3270_DEVICE  : p = "CICS_EPI_ERR_NOT_3270_DEVICE"  ; break;
    case CICS_EPI_ERR_ALREADY_INSTALLED: p = "CICS_EPI_ERR_ALREADY_INSTALLED"; break;
    case CICS_EPI_ERR_USERID_INVALID   : p = "CICS_EPI_ERR_USERID_INVALID"   ; break;
    case CICS_EPI_ERR_PASSWORD_INVALID : p = "CICS_EPI_ERR_PASSWORD_INVALID" ; break;
    case CICS_EPI_ERR_NULL_USERID      : p = "CICS_EPI_ERR_NULL_USERID"      ; break;
    case CICS_EPI_ERR_NULL_PASSWORD    : p = "CICS_EPI_ERR_NULL_PASSWORD"    ; break;
    case CICS_EPI_ERR_CCSID_INVALID    : p = "CICS_EPI_ERR_CCSID_INVALID"    ; break;
    case CICS_EPI_ERR_SERVER_BUSY      : p = "CICS_EPI_ERR_SERVER_BUSY"      ; break;
    case CICS_EPI_ERR_ABENDED          : p = "CICS_EPI_ERR_ABENDED"          ; break;
    case CICS_EPI_ERR_RESOURCE_SHORTAGE: p = "CICS_EPI_ERR_RESOURCE_SHORTAGE"; break;
    case CICS_EPI_ERR_NO_CICS          : p = "CICS_EPI_ERR_NO_CICS"          ; break;
    case CICS_EPI_ERR_MAX_SYSTEMS      : p = "CICS_EPI_ERR_MAX_SYSTEMS"      ; break;
    case CICS_EPI_ERR_MAX_SESSIONS     : p = "CICS_EPI_ERR_MAX_SESSIONS"     ; break;
    default:
        sprintf(AsciiRc, "Unknown (%d)", Rc);
        p = AsciiRc;
        break;
    }
    printf("Unexpected EPI return code: %s\n", p);
    fflush(stdout);
}


/* Function: EventType

   Purpose:  Display the event type returned from CICS_EpiGetEvent */

char *EventType(CICS_EpiEvent_t event)
{
    switch(event)
    {
    case CICS_EPI_EVENT_SEND     : return "CICS_EPI_EVENT_SEND"     ;
    case CICS_EPI_EVENT_CONVERSE : return "CICS_EPI_EVENT_CONVERSE" ;
    case CICS_EPI_EVENT_END_TRAN : return "CICS_EPI_EVENT_END_TRAN" ;
    case CICS_EPI_EVENT_START_ATI: return "CICS_EPI_EVENT_START_ATI";
    case CICS_EPI_EVENT_END_TERM : return "CICS_EPI_EVENT_END_TERM" ;
    case CICS_EPI_EVENT_ADD_TERM : return "CICS_EPI_EVENT_ADD_TERM" ;
    default:
        sprintf(AsciiEvent, "Unknown event (%d)", event);
        return AsciiEvent;
    }
}


/* Function: EndReason

   Purpose:  Display the end reason after end of transaction */

char *EndReason(CICS_EpiEnd_t endReason)
{
    switch(endReason)
    {
    case CICS_EPI_TRAN_NO_ERROR      : return "CICS_EPI_TRAN_NO_ERROR"      ;
    case CICS_EPI_TRAN_NOT_STARTED   : return "CICS_EPI_TRAN_NOT_STARTED"   ;
    case CICS_EPI_TRAN_STATE_UNKNOWN : return "CICS_EPI_TRAN_STATE_UNKNOWN" ;
    case CICS_EPI_READTIMEOUT_EXPIRED: return "CICS_EPI_READTIMEOUT_EXPIRED";
    case CICS_EPI_END_SIGNOFF        : return "CICS_EPI_END_SIGNOFF"        ;
    case CICS_EPI_END_SHUTDOWN       : return "CICS_EPI_END_SHUTDOWN"       ;
    case CICS_EPI_END_OUTSERVICE     : return "CICS_EPI_END_OUTSERVICE"     ;
    case CICS_EPI_END_UNKNOWN        : return "CICS_EPI_END_UNKNOWN"        ;
    case CICS_EPI_END_FAILED         : return "CICS_EPI_END_FAILED"         ;
    default:
        sprintf(AsciiReason, "Unknown reason (%d)", endReason);
        return AsciiReason;
    }
}


/* Function: CallBackProc

   Purpose:  For an ASYNC call, notify the waiting thread that an
             EPI event has arrived */

void CICSEXIT CallBackProc (cics_ushort_t termIndex)
{
    if(FlagAnEvent( &Callback ) != CCLTRUE)
    {
        printf("%d : Failed to post callback flag for %d - test will fail\n",
               QueryMyThreadId(), termIndex);
        fflush(stdout);
    }
    return;
}


/* Function: DumpDataStream

   Purpose:  Display 3270 datastream as hexadecimal and ASCII */

void DumpDataStream(cics_char_t *Start, cics_ushort_t Size)
{
    int row, column;

    for(row = 0; row*16 < Size; row++)
    {
        printf("     ");
        for(column = 0; column < 16; column++)
        {
            if(row*16 + column < Size)
                printf("%2.2X", (unsigned char)Start[row*16 + column]);
            else
                printf("  ");
        }
        printf("   ");
        for(column = 0; row*16 + column < Size && column < 16; column++)
        {
            if(isprint(Start[row*16 + column]))
                printf("%c", Start[row*16 + column]);
            else
                printf(".");
        }
        printf("\n");
    }
}


/* Function: GetScreen

   Purpose:  Get EPI events until end of transaction; return the
             last datastream, and the next transaction name */

cics_sshort_t GetScreen(cics_ushort_t termIndex, cics_char_t *TransId,
                        cics_ubyte_t *DataPtr, cics_ushort_t *DataSize)
{
    cics_sshort_t       rc;
    CICS_EpiEventData_t myEventData;
    cics_ushort_t       ReturnedStreamSize = 0;
    int                 loop;


    /* Get events until we get an END_TRAN event */

    loop = CCLTRUE;
    do
    {
        myEventData.Data = DataPtr;
        myEventData.Size = *DataSize;

        rc = CICS_EpiGetEvent(termIndex, CICS_EPI_WAIT, &myEventData);
        if(rc != CICS_EPI_NORMAL && rc != CICS_EPI_ERR_MORE_EVENTS)
        {
            return rc;
        }

        if(myEventData.Event == CICS_EPI_EVENT_END_TRAN)
        {
            loop = CCLFALSE;
        } else {

          /* Output reply from CICS. */

          printf("  Event type %s received from CICS\n",
                 EventType(myEventData.Event));

          if (Debug == CCLTRUE) {
            printf("  Data stream returned:\n");
            DumpDataStream((cics_char_t *)myEventData.Data, myEventData.Size);
          } else {
            printf("  Data returned: %s\n",
                           (cics_char_t *)(myEventData.Data+2));
          }

          ReturnedStreamSize = myEventData.Size;
        }

    } while(loop == CCLTRUE);

    printf("End of transaction; end reason %s\n",
           EndReason(myEventData.EndReason));
    if(myEventData.EndReason != CICS_EPI_TRAN_NO_ERROR)
    {
        return myEventData.EndReturnCode;
    }
    strcpy(TransId, (char *)&myEventData.TransId);
    *DataSize = ReturnedStreamSize;

    fflush(stdout);
    return rc;
}
