/* File Name(s)  : esib1.c

   Product       : CICS Transaction Gateway

   Description   : This sample shows the basic principles of the ESI API. It
                   takes user supplied values for UserID and Password and
                   verifies them on a chosen server, then calls CICS program
                   EC01 and displays the result.

   Prerequisites:  This sample requires the supplied EC01 transaction to be
                   installed on your CICS server.

 Licensed Materials - Property of IBM  
  
 5724-I81,5725-B65 
  
 (C) Copyright IBM Corp. 1999, 2014 All Rights Reserved.  
  
 US Government Users Restricted Rights - Use, duplication or  
 disclosure restricted by GSA ADP Schedule Contract with  
 IBM Corp.  
    
    Status: Version 9 Release 2 

                   The following code is sample code created by IBM
                   Corporation.  This sample code is not part of any standard
                   IBM product and is provided to you solely for the purpose
                   of assisting you in the development of your applications.
                   The code is provided 'AS IS', without warranty or
                   condition of any kind. IBM shall not be liable for any
                   damages arising out of your use of the sample code, even
                   if IBM has been advised of the possibility of such damages. */


/* Include files */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#include <cics_eci.h>
#include <cics_esi.h>


/* Structures, defines and macro definitions */

#define ECI_SYNC_SIZE 18

#define copyright_notice copyright4cesib1
char version4cesib1[] =
"\nCICS Transaction Gateway"
" Version 9.2.0.3"
" Build Level c920-20180312-1735" ;

char copyright_notice[] =
"\n"
"Licensed Materials - Property of IBM. "
"\n"
"5724-I81,5725-B65 "
"\n"
"(c) Copyright IBM Corp. 1999, 2014 All Rights Reserved. "
"\n"
"US Government Users Restricted Rights - Use, duplication or "
"disclosure restricted by GSA ADP Schedule Contract with "
"IBM Corp."
"\n";


/* Function prototypes */

void Response(int rc, char *Abend);


/* Main Function */

int main(){
   CICS_EciSystem_t servers[20];             /*Servers from CICS_EciListSystems  */
   cics_ushort_t    serverCount    = 20;     /*Length counter for above list     */
   cics_char_t      program[4 + 1] = "EC01"; /*Program to call                   */
   cics_char_t      chosenServer[CICS_ESI_SYSTEM_MAX +1]; /*Selected server name */
   ECI_PARMS        EciParms;                /*The ECI Parameter block           */
   char             CommArea[ECI_SYNC_SIZE]; /*COMMAREA for use in ECI calls     */

   CICS_EsiDetails_t  esiInfo;               /*The information returned by ESI   */
   char               UserID[CICS_ESI_USERID_MAX+1]   = "sysad   ";
   char               PassWd[CICS_ESI_PASSWORD_MAX+1] = "sysad   ";

   char             tempstr[255];

   int              rc, i, choice=0;
   int              loop=0;


   /* Display Banner */

   printf("CICS Transaction Gateway Basic ESI Sample\n\n");

   printf("Servers defined in the Client daemon:\n");


   /* Ask the Client daemon for a list of servers it has defined */

   rc = CICS_EciListSystems(NULL,&serverCount,servers);
   if (rc == ECI_NO_ERROR) {
      do {

         /* Display the list and ask for user input to choose one */

         for (i=0; i<serverCount;i++) {
            printf("   %d.%s\n",i+1 , servers[i].SystemName);
         }

         printf("\nChoose server to connect to:\n");
         scanf("%s", tempstr);
         choice = atol(tempstr);


         /* Validate input and redisplay the list if invalid */

         if (choice > 0 && choice < serverCount+1) {
            strcpy(chosenServer,servers[choice-1].SystemName);
            loop=0;
         } else {
            loop=1;
         }
      }while (loop==1);


      /* Get user ID and password to verify on server */

      printf("\nEnter your user ID for verification:\n");
      scanf("%s", UserID);
      printf("Enter password for user ID %s:\n", UserID);
      scanf("%s", PassWd);


      /* Use ESI to check if password is valid */

      printf("Verifying password\n");
      rc = CICS_VerifyPassword(UserID, PassWd, chosenServer, &esiInfo);
      if (rc == CICS_ESI_NO_ERROR) {


         /* Print out information on ESI call */

         printf("Password verified\n\n");
         printf("User ID and password last verified on: %d/%d/%d at %02d:%02d:%02d\n",
                esiInfo.LastVerifiedDate.Day,
                esiInfo.LastVerifiedDate.Month,
                esiInfo.LastVerifiedDate.Year,
                esiInfo.LastVerifiedTime.Hours,
                esiInfo.LastVerifiedTime.Minutes,
                esiInfo.LastVerifiedTime.Seconds);
         printf("Password expires on: %d/%d/%d at %02d:%02d:%02d\n\n",
                esiInfo.ExpiryDate.Day,
                esiInfo.ExpiryDate.Month,
                esiInfo.ExpiryDate.Year,
                esiInfo.ExpiryTime.Hours,
                esiInfo.ExpiryTime.Minutes,
                esiInfo.ExpiryTime.Seconds);


         /* Continue with a normal ECI call using given security details
            Set up the ECI Parameter block */

         memset(CommArea, '\0', ECI_SYNC_SIZE);
         memset(&EciParms, 0, sizeof(ECI_PARMS));

         EciParms.eci_version                           = ECI_VERSION_1A;
         EciParms.eci_call_type                         = ECI_SYNC;
         memcpy(&EciParms.eci_program_name, program,      4);
         memcpy(&EciParms.eci_userid,       UserID,       8);
         memcpy(&EciParms.eci_password,     PassWd,       8);
         memcpy(&EciParms.eci_system_name,  chosenServer, 8);
         EciParms.eci_commarea                          = CommArea;
         EciParms.eci_commarea_length                   = ECI_SYNC_SIZE;
         EciParms.eci_extend_mode                       = ECI_NO_EXTEND;
         EciParms.eci_luw_token                         = ECI_LUW_NEW;
         EciParms.eci_timeout                           = 0;


         /* Call the chosen server with the given parameters */

         rc = CICS_ExternalCall (&EciParms);
         if (rc == ECI_NO_ERROR) {
            printf("Program %s returned with data: %s\n", program, CommArea);
         } else {
            Response(rc, EciParms.eci_abend_code);
         }

      } else if (rc == CICS_ESI_ERR_PASSWORD_EXPIRED) {
         printf("Password expired\n");
      } else {
         printf("Verification failed\n");
         Response(rc, NULL);
         return 1;
      }

   } else {
      Response(rc, NULL);
      return 1;
   }

   return 0;
}


/* Function: Response

   Purpose:  Display the immediate response code from an ECI call */

void Response(int Rc, char *Abend){
   char *p;
   char *s;
   char *abendString;

   switch (Rc) {
   case ECI_NO_ERROR:                p = "ECI_NO_ERROR               ";s = "ECI";break;
   case ECI_ERR_INVALID_DATA_LENGTH: p = "ECI_ERR_INVALID_DATA_LENGTH";s = "ECI";break;
   case ECI_ERR_INVALID_EXTEND_MODE: p = "ECI_ERR_INVALID_EXTEND_MODE";s = "ECI";break;
   case ECI_ERR_NO_CICS:             p = "ECI_ERR_NO_CICS            ";s = "ECI";break;
   case ECI_ERR_CICS_DIED:           p = "ECI_ERR_CICS_DIED          ";s = "ECI";break;
   case ECI_ERR_REQUEST_TIMEOUT:     p = "ECI_ERR_REQUEST_TIMEOUT    ";s = "ECI";break;
   case ECI_ERR_RESPONSE_TIMEOUT:    p = "ECI_ERR_RESPONSE_TIMEOUT   ";s = "ECI";break;
   case ECI_ERR_TRANSACTION_ABEND:   p = "ECI_ERR_TRANSACTION_ABEND  ";s = "ECI";break;
   case ECI_ERR_LUW_TOKEN:           p = "ECI_ERR_LUW_TOKEN          ";s = "ECI";break;
   case ECI_ERR_SYSTEM_ERROR:        p = "ECI_ERR_SYSTEM_ERROR       ";s = "ECI";break;
   case ECI_ERR_NULL_WIN_HANDLE:     p = "ECI_ERR_NULL_WIN_HANDLE    ";s = "ECI";break;
   case ECI_ERR_NULL_MESSAGE_ID:     p = "ECI_ERR_NULL_MESSAGE_ID    ";s = "ECI";break;
   case ECI_ERR_THREAD_CREATE_ERROR: p = "ECI_ERR_THREAD_CREATE_ERROR";s = "ECI";break;
   case ECI_ERR_INVALID_CALL_TYPE:   p = "ECI_ERR_INVALID_CALL_TYPE  ";s = "ECI";break;
   case ECI_ERR_ALREADY_ACTIVE:      p = "ECI_ERR_ALREADY_ACTIVE     ";s = "ECI";break;
   case ECI_ERR_RESOURCE_SHORTAGE:   p = "ECI_ERR_RESOURCE_SHORTAGE  ";s = "ECI";break;
   case ECI_ERR_NO_SESSIONS:         p = "ECI_ERR_NO_SESSIONS        ";s = "ECI";break;
   case ECI_ERR_NULL_SEM_HANDLE:     p = "ECI_ERR_NULL_SEM_HANDLE    ";s = "ECI";break;
   case ECI_ERR_INVALID_DATA_AREA:   p = "ECI_ERR_INVALID_DATA_AREA  ";s = "ECI";break;
   case ECI_ERR_INVALID_VERSION:     p = "ECI_ERR_INVALID_VERSION    ";s = "ECI";break;
   case ECI_ERR_UNKNOWN_SERVER:      p = "ECI_ERR_UNKNOWN_SERVER     ";s = "ECI";break;
   case ECI_ERR_CALL_FROM_CALLBACK:  p = "ECI_ERR_CALL_FROM_CALLBACK ";s = "ECI";break;
   case ECI_ERR_MORE_SYSTEMS:        p = "ECI_ERR_MORE_SYSTEMS       ";s = "ECI";break;
   case ECI_ERR_NO_SYSTEMS:          p = "ECI_ERR_NO_SYSTEMS         ";s = "ECI";break;
   case ECI_ERR_SECURITY_ERROR:      p = "ECI_ERR_SECURITY_ERROR     ";s = "ECI";break;
   case ECI_ERR_MAX_SYSTEMS:         p = "ECI_ERR_MAX_SYSTEMS        ";s = "ECI";break;
   case ECI_ERR_MAX_SESSIONS:        p = "ECI_ERR_MAX_SESSIONS       ";s = "ECI";break;
   case ECI_ERR_ROLLEDBACK:          p = "ECI_ERR_ROLLEDBACK         ";s = "ECI";break;
   case CICS_ESI_ERR_NULL_USERID:         p = "CICS_ESI_ERR_NULL_USERID        ";s = "ESI";break;
   case CICS_ESI_ERR_NULL_OLD_PASSWORD:   p = "CICS_ESI_ERR_NULL_OLD_PASSWORD  ";s = "ESI";break;
   case CICS_ESI_ERR_NULL_NEW_PASSWORD:   p = "CICS_ESI_ERR_NULL_NEW_PASSWORD  ";s = "ESI";break;
   case CICS_ESI_ERR_PEM_NOT_SUPPORTED:   p = "CICS_ESI_ERR_PEM_NOT_SUPPORTED  ";s = "ESI";break;
   case CICS_ESI_ERR_PEM_NOT_ACTIVE:      p = "CICS_ESI_ERR_PEM_NOT_ACTIVE     ";s = "ESI";break;
   case CICS_ESI_ERR_PASSWORD_EXPIRED:    p = "CICS_ESI_ERR_PASSWORD_EXPIRED   ";s = "ESI";break;
   case CICS_ESI_ERR_PASSWORD_INVALID:    p = "CICS_ESI_ERR_PASSWORD_INVALID   ";s = "ESI";break;
   case CICS_ESI_ERR_PASSWORD_REJECTED:   p = "CICS_ESI_ERR_PASSWORD_REJECTED  ";s = "ESI";break;
   case CICS_ESI_ERR_USERID_INVALID:      p = "CICS_ESI_ERR_USERID_INVALID     ";s = "ESI";break;
   default:
      {
         p = "!!!Unknown!!!";
         s = "!!!Unknown!!!";
         printf("Unknown return code : %d\n", Rc);
      }
      break;
   }

   printf("%s returned: %s\n", s, p);
   if (Rc != ECI_NO_ERROR && s[1] != 'S') {
      if (Abend==NULL) {
         abendString="NULL";
      } else {
         abendString=Abend;
      }
      printf("Abend code was \"%4.4s\"\n", abendString);
   }

}
