/*
   MODULE NAME        ctgclient_eci.h

   DESCRIPTIVE NAME   CICS Transaction Gateway ECI and ESI Version 2.3
                      Application Programming Interface, ECI Parts

   Service level: V9.2.0.3  
 Licensed Materials - Property of IBM  
  
 "Restricted Materials of IBM"  
  
 5724-I81,5725-B65,5655-Y20 
  
 (C) Copyright IBM Corp. 2008, 2014  
  
 US Government Users Restricted Rights - Use, duplication or  
 disclosure restricted by GSA ADP Schedule Contract with  
 IBM Corp.  

   Status: Version 9 Release 2 

   NOTES :-

   This header file is provided with the CICS Transaction Gateway. These
   products are available for a number of different operating environments
   and to ensure the correct sections of this header are included, one of
   the following constants must be defined prior to inclusion:

     CICS_W32  -  if building a Windows application
     CICS_AIX  -  if building an AIX application
     CICS_SOL  -  if building a Solaris application
     CICS_LNX  -  if building a Linux application
     CICS_HPIT and CICS_HPUX -
                  if building an HPUX application on Itanium
*/
/**
* @file
* Defines functions and typedefs for sending External Call Interface
* (ECI) requests to a Gateway daemon.
*/

#ifndef _CTG_ECI_H
#define _CTG_ECI_H

#include "ctgclient.h"
#include "eci_channel.h"

#if defined (CICS_W32) || defined(CICS_WNT)
  #define CICSCALL __cdecl
#else
  #define CICSCALL
#endif

#if !defined(_LP64) && !defined(__LP64__) && !defined(_WIN64) && !defined(__64BIT__)
#if defined(CICS_AIX)
#pragma options align=packed
#elif defined(CICS_HPUX)
#pragma pack 1
#else
#pragma pack(1)
#endif
#endif

/*****************************************************************************/
/** The Version 2 External Call Interface (ECI) parameter block.
 *  The CTG_ECI_PARMS structure is the parameter block used when communicating
 *  with a CICS server. The parameter block fields are used for input and
 *  output. To communicate with the CICS server through the Gateway daemon use
 *  the CTG_ECI_Execute() or CTG_ECI_Execute_Channel() function.
 *
 *  Set the ECI parameter block to nulls before setting the input parameter
 *  fields.
 *
 *  Guidance on how to use the ECI to manage logical units of work can be
 *  found in the product documentation, search for
 *  "Managing logical units of work".
 *
 *  The ECI parameter block is transmitted to the Gateway daemon without any
 *  code page conversion. For more information on managing code pages refer
 *  to the product documentation, search for title "Data conversion".
 *
 *  \note Text strings are NOT null-terminated.
 *
 */
typedef struct
{
   /* Values used with eci_call_type, for detailed information refer to the
    * data field documentation for eci_call_type */
   /**
    * A value used with the CTG_ECI_PARMS::eci_call_type field.
    */
   #define ECI_SYNC                     1
   /**
    * A value used with the CTG_ECI_PARMS::eci_call_type field.
    */
   #define ECI_ASYNC                    2
   /**
    * A value used with the CTG_ECI_PARMS::eci_call_type field.
    */
   #define ECI_GET_REPLY                3
   /**
    * A value used with the CTG_ECI_PARMS::eci_call_type field.
    */
   #define ECI_GET_REPLY_WAIT           4
   /**
    * A value used with the CTG_ECI_PARMS::eci_call_type field.
    */
   #define ECI_GET_SPECIFIC_REPLY       5
   /**
    * A value used with the CTG_ECI_PARMS::eci_call_type field.
    */
   #define ECI_GET_SPECIFIC_REPLY_WAIT  6

	/** \brief Identifies the type of ECI flow.
    *
    *  A required input parameter. The call type identifies the type of the ECI
    *  flow. Asynchronous call types require the eci_version field to be set to
    *  #ECI_VERSION_2A. The valid values are:
    *
    *  \li #ECI_SYNC<br>
    *  The ECI_SYNC call type provides a synchronous program link call to
    *  start, continue, or complete a logical unit of work. Control is
    *  returned to the application once the called CICS program has run to
    *  completion.
    *  \li #ECI_ASYNC<br>
    *  The ECI_ASYNC call type provides and asynchronous program link call to
    *  start, continue, or complete a logical unit of work. Control is returned
    *  to the application once the ECI request has been sent to the Gateway
    *  daemon.
    *  \li #ECI_GET_REPLY<br>
    *  The ECI_GET_REPLY call type provides a reply solicitation call that
    *  returns one of the available replies. If there is no reply available,
    *  #ECI_ERR_NO_REPLY is returned.
    *  \li #ECI_GET_REPLY_WAIT<br>
    *  The ECI_GET_REPLY_WAIT call type provides a reply solicitation call
    *  that returns one of the available replies. If there is no reply
    *  available, the call waits until a reply is available.
    *  \li #ECI_GET_SPECIFIC_REPLY<br>
    *  The ECI_GET_SPECIFIC_REPLY call type provides a reply solicitation
    *  call that returns the reply for the asynchronous request identified
    *  by eci_message_qualifier. If there is no reply available,
    *  ECI_ERR_NO_REPLY is returned.
    *  \li #ECI_GET_SPECIFIC_REPLY_WAIT<br>
    *  The ECI_GET_SPECIFIC_REPLY_WAIT call type provides a reply solicitation
    *  call that returns the reply for the asynchronous request identified by
    *  eci_message_qualifier. If there is no reply available, the call waits
    *  until the reply is available.
    */
   signed short   eci_call_type;

   /**
    * Length of the eci_program_name field.
    */
   #define ECI_PROGRAM_NAME_LENGTH 8
   /** \brief  The CICS program to run.
    *  An input parameter, the CICS program name is required except when
    *  eci_extend_mode is #ECI_COMMIT or #ECI_BACKOUT.
    *
    *  An 8 character field containing the name of the program to be called.
    *  Pad unused characters with spaces or nulls. To avoid code page
    *  conversion issues it is recommended to use uppercase characters A to Z,
    *  and the numeric characters 0 to 9. For more information on managing code
    *  pages refer to the product documentation, search
    *  for the title "Data conversion". */
   char  eci_program_name[ECI_PROGRAM_NAME_LENGTH];

   /**
    * Length of the eci_userid field.
    */
   #define ECI_USERID_LENGTH       16
   /** \brief User ID field provided for compatibility with existing
    * applications. For new applications, use the eci_userid_ptr field; this
    * supports user IDs longer than 16 characters. */
   char  eci_userid[ECI_USERID_LENGTH];

   /**
    * Length of the eci_password field.
    */
   #define ECI_PASSWORD_LENGTH     16
   /** \brief Password field provided for compatibility with existing
    * applications. For new applications, use the eci_password_ptr field; this
    * supports password phrases longer than 16 characters. */
   char  eci_password[ECI_PASSWORD_LENGTH];

   /**
    * Length of the eci_transid field.
    */
   #define ECI_TRANSID_LENGTH      4
   /** \brief A CICS transaction identifier.
    *
    *  A 4 character field optionally containing the ID of a CICS transaction.
    *  Pad unused characters with spaces or nulls. The parameter is ignored if
    *  eci_tpn is used. The value of eci_transid is stored in the CICS server
    *  EIBTRNID field for the duration of the LINK to the program specified in
    *  the eci_program_name.
    *
    *  The called program runs under the default CICS mirror transaction, but
    *  is linked to under the eci_transid transaction name. This name is
    *  available to the called program for querying the transaction ID. Some
    *  servers use the transaction ID to determine security and performance
    *  attributes for the called program. In those servers, use this parameter
    *  to control the processing of your called programs. */
   char  eci_transid[ECI_TRANSID_LENGTH];

   /**
    * Length of the eci_abend_code field.
    */
   #define ECI_ABEND_CODE_LENGTH   4
   /** \brief CICS abend code.
   *
   *   An output parameter, a 4 character field in which a CICS abend code is
   *   returned. If the transaction that executes the called program does not
   *   abend, the field contains space characters on return. */
   char  eci_abend_code[ECI_ABEND_CODE_LENGTH];

   /** \brief The COMMAREA data passed to and returned from the CICS program.
    *
    *  An optional input parameter, a pointer to the data to be passed in the
    *  COMMAREA to the CICS program identified by eci_program_name. If no
    *  COMMAREA is required, supply a null pointer and set eci_commarea_length
    *  to zero. The COMMAREA is also an output field, it can be used by the
    *  called program to return information to the application. A COMMAREA can
    *  only be returned from the called program, if one was sent. */
   void *         eci_commarea;

   /** \brief The length of the COMMAREA.
    *
    *  The maximum length of the COMMAREA in bytes. The COMMAREA lengths for
    *  sending and receiving data can be defined using the
    *  commarea_outbound_length and commarea_inbound_length parameters.
    *
    *  Use a maximum size of 32500 bytes, as this is guaranteed to be flowed
    *  successfully across all protocols. COMMAREA sizes greater than this may
    *  generate an ECI_ERR_INVALID_DATA_LENGTH error. */
   signed short   eci_commarea_length;

   /** \brief The ECI request timeout.
   *
   *   The time in seconds to wait for a response from the CICS server. A value
   *   of zero means that no time limit is set.
   *
   *   An ECI timeout value of zero indicates that this ECI request will
   *   not be timed out by CICS Transaction Gateway. An ECI timeout value
   *   greater than zero indicates that this ECI request might be timed out by
   *   CICS Transaction Gateway. When an ECI timeout expires, the associated
   *   unit of work in CICS might have already completed or might have been
   *   backed out. This information is not available to the client application.
   *
   *   If using remote mode IPIC connections, this value can be overridden by
   *   the ECITIMEOUT parameter on the IPIC server definition in the CICS
   *   Transaction Gateway configuration file. */
   signed short   eci_timeout;

   /* Values used with eci_extend_mode, for detailed information refer to the
    * data filed documentation for eci_extend_mode */
   /**
    * A value used with the CTG_ECI_PARMS::eci_extend_mode field.
    */
   #define ECI_NO_EXTEND        0
   /**
    * A value used with the CTG_ECI_PARMS::eci_extend_mode field.
    */
   #define ECI_EXTENDED         1
   /**
    * A value used with the CTG_ECI_PARMS::eci_extend_mode field.
    */
   #define ECI_COMMIT           2
   /**
    * A value used with the CTG_ECI_PARMS::eci_extend_mode field.
    */
   #define ECI_BACKOUT          4

   /** \brief Extend mode defines the behaviour of the ECI flow.
   *
   *  A required input parameter that defines how a logical unit of work is
   *  processed.
   *
   *  The valid values are:
   *  <UL>
   *  <li>     #ECI_NO_EXTEND
   *   <OL><li> If the eci_luw_token is zero, this call is the only call for
   *            the logical unit of work.
   *       <li> If the eci_luw_token is not zero, this call is the last call
   *            for the specified logical unit of work.
   *  </OL>
   *  In each case, if no error occurs, changes to recoverable resources are
   *  committed by a CICS end-of-task sync point, and the logical unit of work
   *  ends. If an error occurs, changes to recoverable resources are backed out
   *  and the logical unit of work ends.
   *
   *  <li>      #ECI_EXTENDED
   *   <OL><li>  If the eci_luw_token is zero, this call is the first call for
   *             a logical unit of work that is to be continued.
   *       <li>  If the eci_luw_token is not zero, this call continues the
   *             specified logical unit of work.
   *  </OL>
   *  In each case, the logical unit of work continues and changes to
   *  recoverable resources remain uncommitted.
   *
   *   <li>      #ECI_COMMIT
   *  \n Terminates the logical unit of work, identified by the eci_luw_token,
   *  and commits all changes made to recoverable resources.
   *
   *   <li>      #ECI_BACKOUT
   *  \n Terminates the logical unit of work, identified by the eci_luw_token,
   *  and backs out all changes made to recoverable resources.
   *  </UL>
   *
   *  For guidance on how to use the ECI to manage logical units of work, refer
   *  to the product documentation, search for "Managing logical units of work".
   */
   signed short   eci_extend_mode;

   /**
    * A value used to set the CTG_ECI_PARMS::eci_luw_token field at the start of
    * a new logical unit of work.
    */
   #define  ECI_LUW_NEW 0
   /**  \brief The token identifies the logical unit of work to which the ECI
    * request is associated.
    *
    * An integer field used for identifying the logical unit of work to which
    * a call belongs. The eci_luw_token must be set to #ECI_LUW_NEW at the
    * start of a logical unit of work, regardless of whether the logical unit of
    * work is going to be extended. If the logical unit of work is to be
    * extended, the ECI updates eci_luw_token with a valid value on the first
    * call of the logical unit of work, the returned value must be used as the
    * input value for all subsequent calls that are part of the same logical
    * unit of work.
    *
    * If the return code is not #ECI_NO_ERROR, and the call was continuing or
    * ending an existing logical unit of work, this field is used as output to
    * report the condition of the logical unit of work. If it is set to zero the
    * logical unit of work has ended and its updates have been backed out. If it
    * is nonzero, it is the same as the input value, the logical unit of work is
    * continuing, and its updates are still pending. The LUW token must be set
    * to zero when eci_extend_mode is #ECI_NO_EXTEND */
   signed int     eci_luw_token;

   /** ECI version provided for applications that use the eci_userid and
    * eci_password fields to specify security credentials. For new applications,
    * specify an eci_version of #ECI_VERSION_2A and use the eci_userid_ptr and
    * eci_password_ptr fields for security credentials. ECI_VERSION_2 supports
    * only the #ECI_SYNC call type.
    * */
   #define ECI_VERSION_2        4

   /** Latest ECI version. ECI_VERSION_2A supports both synchronous and
    * asynchronous ECI call types. Security credentials are specified using
    * eci_userid_ptr and eci_password_ptr.
    */
   #define ECI_VERSION_2A       6

   /** \brief ECI version identifier.
    *
    *  A required input parameter that identifies the version of the ECI, for
    *  which the application is coded, to CICS Transaction Gateway. This field
    *  must be set to #ECI_VERSION_2 or #ECI_VERSION_2A. #ECI_VERSION_2 is
    *  provided for compatibility with existing applications that use the
    *  #ECI_SYNC call type and the eci_userid and eci_password fields.
    *  #ECI_VERSION_2A must be used for asynchronous call support. */
   signed short   eci_version;

   /**
    * Length of the eci_system_name field.
    */
   #define ECI_SYSTEM_NAME_LENGTH  8
   /** \brief Server name.
   *
   *   An 8 character field that specifies the name of the server to which the
   *   ECI request is to be directed. Pad unused characters with spaces or
   *   nulls. Set the server name for each request that starts a new logical
   *   unit of work. The server name is ignored for subsequent requests in an
   *   extended LUW, as all requests for an extended LUW must be sent to the
   *   same server.
   *
   *   The Gateway daemon may change the server depending on the high
   *   availability configuration. */
   char     eci_system_name[ECI_SYSTEM_NAME_LENGTH];

   /**
    * Length of the eci_tpn field.
    */
   #define ECI_TPN_LENGTH          4
   /** \brief A transaction identifier for the mirror transaction.
    *
    *  Optional input parameter. A 4-character field that specifies the
    *  transaction ID that will be used in the server to process the ECI
    *  request. This transaction must be defined in the server as a CICS mirror
    *  transaction. If the field is not set, the default CICS mirror
    *  transaction is used.
    *
    *  If the ECI request is extended (see the description of eci_extend_mode),
    *  this parameter is used only for the first request. Subsequent requests
    *  within the same unit of work will use the transaction specified on the
    *  first request.
    *
    *  If this field is used, the contents of eci_transid are ignored. */
   char     eci_tpn[ECI_TPN_LENGTH];

   /** \brief Maximum data length to send to the Gateway daemon. If
    *  commarea_outbound_length is set to 0 or is greater than
    *  eci_commarea_length, the value of eci_commarea_length is used instead
    *  of commarea_outbound_length. */
   signed short commarea_outbound_length;

   /** \brief Maximum data length to receive from the Gateway daemon. If the
    * commarea_inbound_length is set to 0 or is greater than
    * eci_commarea_length, the value of eci_commarea_length is used instead of
    * commarea_inbound_length. */
   signed short commarea_inbound_length;

   /** \brief Channel token for use with CTG_ECI_ExecuteChannel().
    * An input parameter for an ECI request invoking a CICS channel program,
    * specify the channel token obtained from ECI_createChannel(). For reply
    * solicitation calls this is an output parameter, used to return the channel
    * token for the returned channel. For an ECI request invoking a CICS
    * COMMAREA program this value is ignored. */
   ECI_ChannelToken_t channel;

   /** \brief Pointer to a null terminated string containing a user ID. To use
    * this field, set eci_version to #ECI_VERSION_2A.
    *
    * Set this field to null if a user ID is not required. The maximum
    * supported field length depends on the CICS server version. For more
    * information see your CICS server documentation. */
   const char * eci_userid_ptr;

   /** \brief Pointer to a null terminated string containing a password or
    * password phrase. To use this field, set eci_version to #ECI_VERSION_2A.
    *
    * Set this field to null if a password or password phrase is not required.
    * The maximum field length depends on the CICS server version and
    * communications protocol type. For more information see your CICS server
    * documentation. */
   const char * eci_password_ptr;

   /**
    * \brief Identifier for an asynchronous request reply.
    *
    * For #ECI_ASYNC calls, this field is an output parameter containing the
    * identifier that can be used to get the response to this specific request.
    *
    * For #ECI_GET_SPECIFIC_REPLY and #ECI_GET_SPECIFIC_REPLY_WAIT calls, this
    * field is an input parameter specifying the identifier of the asynchronous
    * response to be returned.
    *
    * For other call types, this field is not used.
    */
   signed int eci_message_qualifier;

} CTG_ECI_PARMS;


/*****************************************************************************/
/**
 *  A reference for an ECI Parameter block, CTG_ECI_Execute .
 */
typedef CTG_ECI_PARMS *PCTG_ECI_PARMS;


/*****************************************************************************/
/**
 *  Sends an ECI request to the Gateway with a COMMAREA or retrieves an
 *  asynchronous ECI response, depending on the value of
 *  CTG_ECI_PARMS::eci_call_type.
 *
 * @param[in] gwTok The reference to the open Gateway connection.
 * @param EciParms The reference to the CTG_ECI_PARMS structure, the
 * fields of the structure are used for input and output.
 *
 * @returns An integer value identifies whether the call was successful or not.
 * \n Possible values include
 * \li #CTG_OK
 * \li #CTG_ERR_MALLOCFAIL
 * \li #CTG_ERR_NULLGWTOKPTR
 * \li #CTG_ERR_NULLPARM
 * \li #CTG_ERR_LOSTGWCON
 * \li #CTG_ERR_NULLECIPOINTER
 * \li #CTG_ERR_GATEWAY_CLOSED
 * \li #CTG_ERR_WORK_WAS_REFUSED
 * \li #CTG_ERR_GATEWAY_EXCEPTION
 * \li #CTG_ERR_INVALID_REQUEST_TYPE
 * \li #CTG_ERR_SYSTEM_ERROR
 *
 * Other possible values are all of the documented ECI return codes.
 *
*/
int CICSCALL CTG_ECI_Execute(CTG_ConnToken_t gwTok,
                             CTG_ECI_PARMS *EciParms);


/**
 *  Sends an ECI request to the Gateway with a channel object or retrieves an
 *  asynchronous ECI response, depending on the value of
 *  CTG_ECI_PARMS::eci_call_type.
 *
 * @param[in] gwTok The reference to the open Gateway connection.
 * @param EciParms The reference to the CTG_ECI_PARMS structure, the
 * fields of the structure are used for input and output.
 *
 * @returns An integer value identifies whether the call was successful or not.
 * \n Possible values include
 * \li #CTG_OK
 * \li #CTG_ERR_MALLOCFAIL
 * \li #CTG_ERR_NULLGWTOKPTR
 * \li #CTG_ERR_NULLPARM
 * \li #CTG_ERR_LOSTGWCON
 * \li #CTG_ERR_NULLECIPOINTER
 * \li #CTG_ERR_GATEWAY_CLOSED
 * \li #CTG_ERR_WORK_WAS_REFUSED
 * \li #CTG_ERR_GATEWAY_EXCEPTION
 * \li #CTG_ERR_INVALID_REQUEST_TYPE
 * \li #CTG_ERR_SYSTEM_ERROR
 *
 * Other possible values are all of the documented ECI return codes.
 *
*/
int CICSCALL CTG_ECI_Execute_Channel(CTG_ConnToken_t gwTok,
                                     CTG_ECI_PARMS *EciParms);


/**
 * Writes the contents of internal CICS TG channel and
 * container structures to the current trace destination. The debug information
 * is written irrespective of trace settings.
 *
 * \note The IBM service organization may request that this function is used to
 * aid problem determination.
 *
 * @returns An integer value identifies whether the call was successful or not.
 * \n Possible values include
 * \li #CTG_OK
 * \li #CTG_ERR_MALLOCFAIL
 * \li #CTG_ERR_SYSTEM_ERROR
 */
int CICSCALL CTG_ECI_dumpChannels();


/*****************************************************************************/
/*                      CICS TG ECI Return Code constants                    */
/*****************************************************************************/
/**
 * No error.
 */
#define ECI_NO_ERROR                  0
/**
 * Bad data length. This might be because the protocol used to communicate with
 * CICS is unable to send or receive the specified COMMAREA size.
 */
#define ECI_ERR_INVALID_DATA_LENGTH  -1
/**
 * The eci_extend_mode field is not valid.
 */
#define ECI_ERR_INVALID_EXTEND_MODE  -2
/**
 * Communication with the target CICS server could not be established.
 */
#define ECI_ERR_NO_CICS              -3
/**
 * Communication with the target CICS server was interrupted during
 * the request.
 */
#define ECI_ERR_CICS_DIED            -4
/**
 * The timeout interval expired before the request was sent to
 * CICS, or the value supplied for timeout is negative.
 */
#define ECI_ERR_REQUEST_TIMEOUT      -5
/**
 * The ECI call timed out while waiting for a response.
 */
#define ECI_ERR_RESPONSE_TIMEOUT     -6
/**
 * The CICS transaction abended. The abend code is available in the
 * eci_abend_code field.
 */
#define ECI_ERR_TRANSACTION_ABEND    -7
/**
 * The value in eci_luw_token is incorrect. There is no active unit
 * of work associated with the specified token.
 */
#define ECI_ERR_LUW_TOKEN            -8
/**
 * An internal system error has occurred. If the problem persists,
 * collect application and Gateway daemon trace and contact your IBM
 * service representative.
 */
#define ECI_ERR_SYSTEM_ERROR         -9
/**
 * The eci_call_type field is not valid. This can occur for the following
 * reasons:
 * \li The eci_call_type field is not set to a valid call type value.
 * \li The active CICS request exit rejected the request.
 * \li The request includes a channel and the destination CICS server is
 * not a defined IPIC server.
 */
#define ECI_ERR_INVALID_CALL_TYPE    -14
/**
 * An attempt was made to continue an existing logical unit of
 * work, but there was an outstanding asynchronous call for the
 * same logical unit of work.
 */
#define ECI_ERR_ALREADY_ACTIVE       -15
/**
 * There are insufficient resources to communicate with the target
 * CICS server; details might be available in the CICS TG error log
 * files.
 */
#define ECI_ERR_RESOURCE_SHORTAGE    -16
/**
 * There are insufficient communication resources to communicate with
 * the target CICS server.
 */
#define ECI_ERR_NO_SESSIONS          -17
/**
 * The combination of eci_commarea and eci_commarea_length values is not valid.
 * If eci_commarea is NULL, eci_commarea_length must be zero. If eci_commarea
 * is not NULL, eci_commarea_length must not be zero.
 */
#define ECI_ERR_INVALID_DATA_AREA    -19
/**
 * The eci_version field was not set to #ECI_VERSION_2 or #ECI_VERSION_2A.
 */
#define ECI_ERR_INVALID_VERSION      -21
/**
 * The target CICS server is unknown to the Gateway daemon.
 */
#define ECI_ERR_UNKNOWN_SERVER       -22
/**
 * An invalid combination of user ID and password or password phrase
 * was specified.
 */
#define ECI_ERR_SECURITY_ERROR       -27
/**
 * The CICS TG has reached its limit on the maximum number of CICS
 * servers.
 */
#define ECI_ERR_MAX_SYSTEMS          -28
/**
 * There are insufficient communication resources to complete the
 * request.
 */
#define ECI_ERR_MAX_SESSIONS         -29
/**
 * An attempt was made to commit a logical unit of work, but the
 * CICS server was unable to commit the changes, and backed them out
 * instead.
 */
#define ECI_ERR_ROLLEDBACK           -30
/**
 * No reply. This can occur for the following reasons:
 * \li The eci_call_type field is set to #ECI_GET_REPLY and there are no replies
 * outstanding.
 * \li The eci_call_type field is set to #ECI_GET_SPECIFIC_REPLY and there is no
 * reply for the specified message qualifier.
 */
#define ECI_ERR_NO_REPLY             -32
/**
 * The message qualifier is not in use.
 */
#define ECI_ERR_INVALID_MSG_QUAL     -33

#if !defined(_LP64) && !defined(__LP64__) && !defined(_WIN64) && !defined(__64BIT__)
#if defined(CICS_AIX)
#pragma options align=reset
#elif defined(CICS_HPUX) && defined(__cplusplus)
#pragma pack
#elif defined(CICS_HPUX) && defined(CICS_HPIT)
#pragma pack
#elif defined(CICS_HPUX)
#pragma HP_ALIGN HPUX_NATURAL
#else
#pragma pack()
#endif
#endif

#endif /* _CTG_ECI_H */
