/*              CICS TG Classes

   File name:   cclterm.hpp

   Description: C++ declaration of CICS TG EPI Terminal Class (CclTerminal)

 Licensed Materials - Property of IBM  
  
 "Restricted Materials of IBM"  
  
 5724-I81,5725-B65 
  
 (C) Copyright IBM Corp. 1995, 2014  
  
 US Government Users Restricted Rights - Use, duplication or  
 disclosure restricted by GSA ADP Schedule Contract with  
 IBM Corp.  
   
   Status: Version 9 Release 2 
*/


#ifndef _CCLTERM_
#define _CCLTERM_

class CclTerminal {
friend class CclTerminalI;
public:

// Typedefs and enumerations

  enum           State {               // object state values
                   idle, server, client, discon, error, termDefined, txnTimedOut };
  enum           ATIState {            // ATI state values
                   enabled, disabled };
  enum           signonType {          // signon types
                   signonCapable, signonIncapable, signonUnknown };
  enum           EndTermReason {       // Terminal Ended Reasons
                   signoff, shutdown, outOfService, unknown, failed, notdiscon};


// Constructors/Destructors

// define and install a default terminal

  CCLI           CclTerminal(
                   const char* server=NULL,      // CICS server name
                   const char* devtype=NULL,     // device type
                   const char* netname=NULL );   // network name

// define a terminal supporting extra facilities

  CCLI           CclTerminal(
                   const char* server,                // CICS server name
                   const char* devtype,               // device type
                   const char* netname,               // network name
                   signonType signonCapability,       // signon Capability
                   const char* userid,                // userid
                   const char* password,              // password
                   const unsigned short &readTimeout, // read time out
                   const unsigned short &CCSid);      // codepage id

  CCLI virtual   ~CclTerminal();


// Actions

  CCLI void      send(                 // format and send 3270, start transid
                   CclSession* session,           // session object
                   const char* transid,           // start transaction id
                   const char* startdata=NULL );  // start transaction data
  CCLI void      send(                 // format and send 3270 datastream
                   CclSession* session );         // session object
  CCLI Ccl::Bool poll();               // poll for data from CICS server
  CCLI void      receiveATI(           // listen for ATIs (sync or async)
                   CclSession* session );         // session object
  CCLI void      disconnect();         // disconnect terminal from CICS
  CCLI void      disconnect(Ccl::Bool withPurge); // disconnect terminal from CICS

  CCLI ATIState  queryATI();           // query ATI state
  CCLI void      setATI(               // set ATI state
                   ATIState newstate );
  CCLI void      install(              // install a defined terminal
                   CclSession* session,           // session object
                   const unsigned short &installTimeout);
  CCLI void      alterSecurity(                       // update userid/password
                   const char* newUserid,
                   const char* newPassword);
  CCLI void      makeSecurityDefault(); // set current usrid/pw to client deflt
  CCLI CclSecAttr* verifyPassword();        // verify userid/password at server
  CCLI CclSecAttr* changePassword(
                     const char* newPassword);     // change password at server


// Accessors

  CCLI const char*
                 serverName() const;   // get CICS server name
  CCLI const char*
                 netName() const;      // get CICS terminal network name
  CCLI const char*
                 devType() const;      // get CICS terminal device type
  CCLI CclScreen*
                 screen() const;       // access CclScreen object
  CCLI const char*
                 transID() const;      // get current transaction id (4 chars)

  CCLI State     state() const;        // query current object state
  CCLI Ccl::ExCode
                 exCode() const;       // query object condition code

  CCLI const char*
                 diagnose();           // get error text description
  CCLI long      errorCause() const;   // get system error - cause
  CCLI long      errorValue() const;   // get system error - value
  CCLI const char*                     // most recent exception code
                 exCodeText() const;   //   - text
  CCLI const char*
                 userId() const;       // get current userId
  CCLI const char*
                 password() const;     // get current password
  CCLI unsigned short
                 readTimeout() const;  // get the read timeout value
  CCLI unsigned short
                 CCSid() const;        // get the Codepage id
  CCLI signonType
                 signonCapability() const; // get signon capability
  CCLI const char*
                 termID() const;       // get current terminal id
  CCLI EndTermReason
                 discReason() const;   // query the disconnect reason....

protected:

// Actions

  CCLI virtual CclScreen*
                 createScreen(         // override for CclScreen creation
                   unsigned short rows,           // screen height in rows
                   unsigned short cols );         // screen width in columns

private:
  CclTerminalI*  ivpI;   // pointer to private implementation class
}; // end of class CclTerminal

#endif
