/* File Name(s)  : ecii1.cpp

   Product       : CICS Transaction Gateway

   Description   : This sample shows how to use logical units of work to
                   string together multiple transaction executions. All
                   calls are handled asynchronously to show this method of
                   execution

   Prerequisites:  This sample requires the CICS Program EC02 to be
                   installed on your CICS server for operation.

 Licensed Materials - Property of IBM  
  
 5724-I81,5725-B65 
  
 (C) Copyright IBM Corp. 1999, 2014 All Rights Reserved.  
  
 US Government Users Restricted Rights - Use, duplication or  
 disclosure restricted by GSA ADP Schedule Contract with  
 IBM Corp.  
   
   Status: Version 9 Release 2 

                   The following code is sample code created by IBM
                   Corporation.  This sample code is not part of any
                   standard IBM product and is provided to you solely for
                   the purpose of assisting you in the development of your
                   applications.  The code is provided 'AS IS', without
                   warranty or condition of any kind.  IBM shall not be
                   liable for any damages arising out of your use of the
                   sample code, even if IBM has been advised of the
                   possibility of such damages. */


// Include files

#include <stdlib.h>
#include <string.h>

#if defined(CICS_W32) || defined(CICS_LNX)
#include <iostream>
#include <iomanip>
using namespace std;
#else
#include <iostream.h>
#endif

#include <cicseci.hpp>
#include "cclcalls.h"


// Defines

#define ECI_DATA_LENGTH 80

#define NO_EVENT        0
#define EVENT_EXCEPTION 1
#define EVENT_REPLY     2

#define copyright_notice copyright4cppecii1
char version4cppecii1[] =
"\nCICS Transaction Gateway"
" Version 9.2.0.3"
" Build Level c920-20180312-1735" ;

char copyright_notice[] =
"\n"
"Licensed Materials - Property of IBM. "
"\n"
"5724-I81,5725-B65 "
"\n"
"(c) Copyright IBM Corp. 1999, 2014 All Rights Reserved. "
"\n"
"US Government Users Restricted Rights - Use, duplication or "
"disclosure restricted by GSA ADP Schedule Contract with "
"IBM Corp."
"\n";


// Global Variables

CclEvent EciReply;
CclBuf   sendCommarea("");                   //COMMAREA for use in transactions
int      eventFlag;


// Sub-class the ECI object for custom exception handling.

class MyECI: public CclECI {
public:
   void handleException(CclException exception){
      cout << "A problem has occurred. Diagnostic information follows :-";
      cout << endl;
      cout << exception.diagnose() << endl;
      eventFlag = EVENT_EXCEPTION;
      FlagAnEvent(&EciReply);
   }
};


// Sub-class the Flow object for custom reply handling.

class MyFlow: public CclFlow {
public:


   /* Need to override the constructors so that we can construct objects
      Using these constructors. */

   MyFlow(Ccl::Sync syncType,
          unsigned long stackPages = 3) : CclFlow(syncType, stackPages) {
   }

   MyFlow(Ccl::Sync syncType,
          unsigned long stackPages,
          const unsigned short &timeout) : CclFlow(syncType, stackPages, timeout){
   }

   void handleReply(CclBuf* receivedCommarea){


      /* Output Reply from CICS if not a commit or backout. Note that in
         asynchronous comms, the received COMMAREA is a temporary CclBuf
         object, not the COMMAREA block we passed into the call so we must
         copy the data over. */

      if (receivedCommarea != NULL) {
         cout << "  Reply:" << (char*)receivedCommarea->dataArea() << endl;
         sendCommarea.assign(receivedCommarea->dataAreaLength(),
                             receivedCommarea->dataArea());
      }


      //Set event so that main thread can continue.

      eventFlag = EVENT_REPLY;
      FlagAnEvent(&EciReply);
   }
};


// The main entrypoint

int main(void){
   char chosenServer[8 +1];                     //Server name of users choice.
   char progid[4 +1] = "EC02";                  //Program to be executed.

   char UserID[9] = "sysad";                    //Enter your user ID here.
   char PassWd[9] = "sysad";                    //Enter your password here.

   int i, loop    = 1;
   int choice     = -1;
   char input[80];

   cout << "CICS Transaction Gateway Intermediate C++ ECI V1 Sample" << endl << endl;

   MyECI *pEci;
   pEci = new MyECI;


   // Retrieve list of servers from Client daemon.

   do {
      cout << "\nSelect a Server or Q to End\n" << endl;
      for (int i = 1; i <= pEci->serverCount(); i++  ) {
         cout << "    " << i << ". " << pEci->serverName(i)
              << " - " << pEci->serverDesc(i) << endl;
      }


      // Get user choice for server to be used.

      cout << endl << "Choose Server to connect to:";
      cout.flush();
      if (cin.peek()=='\n') cin.ignore(1,'\n');
      cin.getline(input,79,'\n');

      if (input[0] == 'q' || input[0] == 'Q') {
         delete pEci;
         return 0;
      }
      choice = atoi(input);
   } while (choice < 1 || choice > pEci->serverCount());
   strcpy(chosenServer,pEci->serverName(choice));


   //Create event for use in Asynchronous calls

   CreateAnEvent("EciReply", &EciReply);


   //Open a connection with the chosen server.

   CclConn server(chosenServer, UserID, PassWd);
   MyFlow sflow(Ccl::async);

   cout << "Starting unit of work." << endl;
   CclUOW UOW;


   // Set up COMMAREA for sending and holding the received information

   sendCommarea.setDataLength(ECI_DATA_LENGTH);

   do {
      cout << "  Calling program EC02." << endl;


      // Call Transaction on CICS.

      server.link(sflow, progid, &sendCommarea, &UOW);


      // Wait for something to happen.

      cout << "  Waiting for a reply from CICS." << endl;
      WaitOnAnEvent(&EciReply);
      ResetAnEvent(&EciReply);
      if (eventFlag == EVENT_REPLY) {
         cout << "  To call EC02 again, press A. To close the unit of work press C." << endl;
         if (cin.peek()=='\n') cin.ignore(1,'\n');
         cin.getline(input,79,'\n');

         if (input[0] == 'a' || input[0] == 'A') {
            loop = 1;
         } else {
            loop = 0;
         }
      } else {
         loop = 0;
      }


   } while (loop == 1);

   do {

      // Ask user to commit or backout unit of work

      cout << "Press C to commit or B to backout the unit of work."
           << endl;
      if (cin.peek()=='\n') cin.ignore(1,'\n');
      cin.getline(input,79,'\n');
      if (input[0] == 'c' || input[0] == 'C') {


         // Commit Unit of Work.

         cout << "Committing unit of work on " << chosenServer <<  "." << endl;
         UOW.commit(sflow);


         // Wait for the Commit to complete

         cout << "Waiting for a reply from CICS." << endl;
         WaitOnAnEvent(&EciReply);
         if (eventFlag == EVENT_EXCEPTION) {
            cout << "Failed to commit the unit of work!!!" << endl;
         } else {
            cout << "Commit complete." << endl;
         }
         loop = 0;
      } else if (input[0] == 'b' || input[0] == 'B') {


         // Backout Unit of Work.

         cout << "Backing out unit of work on " << chosenServer << "." << endl;
         UOW.backout(sflow);


         // Wait for the backout to complete

         cout << "Waiting for a reply from CICS." << endl;
         WaitOnAnEvent(&EciReply);
         if (eventFlag == EVENT_EXCEPTION) {
            cout << "Failed to backout the unit of work!!!" << endl;
         } else {
            cout << "Backout complete." << endl;
         }
         loop = 0;
      } else {
         loop = 1;
      }
   } while (loop == 1);


   // Destroy Event, delete the Eci object.

   DeleteAnEvent(&EciReply);
   delete pEci;

   return 0;
}
