/*
*         File Name      : BasicCicsPlugin.java
*
*         Product        : CICS Transaction Gateway
*
*         Description    : This sample shows how to implement a CICS  
*                          Intercept plug-in for use with CICS Transaction 
*                          Gateway. The sample plug-in is designed to be used 
*                          with the EciB1 and EciB3 Java, ECI V2 and .NET
*                          samples, and the SC11 scenario included with CICS 
*                          Transaction Gateway.   
*  
*         Pre-requisites : Use a version of the JDK that the CICS Transaction 
*                          Gateway supports if you compile this sample. See   
*                          the product documentation for supported Java levels.
*
* Licensed Materials - Property of IBM  
*  
* 5724-I81,5725-B65,5655-Y20 
*  
* (C) Copyright IBM Corp. 2014, 2016 All Rights Reserved.  
*  
* US Government Users Restricted Rights - Use, duplication or  
* disclosure restricted by GSA ADP Schedule Contract with  
* IBM Corp.  
* 
* Status: Version 9 Release 2 
*
*         The following code is sample code created by IBM Corporation.  This
*         sample code is not part of any standard IBM product and is provided
*         to you solely for the purpose of assisting you in the development of
*         your applications.  The code is provided 'AS IS', without warranty
*         or condition of any kind.  IBM shall not be liable for any damages
*         arising out of your use of the sample code, even if IBM has been
*         advised of the possibility of such damages.
*/


package com.ibm.ctg.samples.intercept;

import java.io.UnsupportedEncodingException;
import java.nio.ByteBuffer;
//import java.nio.ByteOrder;
import java.text.SimpleDateFormat;
import java.util.Date;

import com.ibm.ctg.client.Channel;
import com.ibm.ctg.client.Container;
import com.ibm.ctg.client.ECIRequest;
import com.ibm.ctg.client.ECIReturnCodes;
import com.ibm.ctg.client.GatewayRequest;
import com.ibm.ctg.client.exceptions.ContainerException;
import com.ibm.ctg.client.exceptions.ContainerNotFoundException;
import com.ibm.ctg.intercept.CicsIntercept;

/**
 * Sample {@link CicsIntercept} plug-in that can be used with the {@code EciB1} and
 * {@code EciB3} Java, ECI V2 and .NET samples, and the SC11 scenario included
 * with CICS Transaction Gateway.
 */
public class BasicCicsPlugin implements CicsIntercept {
static final String copyright_notice ="Licensed Materials - Property of IBM 5724-I81,5725-B65,5655-Y20 (c) Copyright IBM Corp. 2014, 2016 All Rights Reserved. US Government Users Restricted Rights - Use, duplication or disclosure restricted by GSA ADP Schedule Contract with IBM Corp."; 

   /**
    * Intercepts the {@link GatewayRequest} before it is sent on to CICS.
    * This plug-in implementation intercepts ECI requests with the program
    * set to EC01 or EC03, mocks the CICS server sample response and returns
    * the request straight to the client. All other requests are allowed to
    * continue through Gateway daemon processing as normal.
    * 
    * @param request the request being flowed
    * @return {@link InterceptAction#Return} for EC01 and EC03 requests,
    *         {@link InterceptAction#Continue} for all other requests.
    * @throws RuntimeException if an error occurs
    */
   public InterceptAction interceptRequest(GatewayRequest request) {
      
      // This plug-in is only designed to handle ECI requests
      if (request instanceof ECIRequest) {
         ECIRequest eciReq = (ECIRequest)request;
         
         if  ((eciReq.getCallType() == ECIRequest.ECI_SYNC)
               && ((eciReq.getExtendMode() == ECIRequest.ECI_NO_EXTEND) || (eciReq.getExtendMode() == ECIRequest.ECI_EXTENDED))) {
            if ("EC01".equals(eciReq.Program)) {
               // Check the Commarea and Commarea_Length fields are consistent
               int commareaLen = eciReq.Commarea_Length;
               if ((commareaLen == 0) && (eciReq.Commarea != null)) {
                  commareaLen = eciReq.Commarea.length;
               } else if ((eciReq.Commarea == null) || (eciReq.Commarea.length < commareaLen)) {
                  eciReq.Commarea = new byte[commareaLen];
               }
               eciReq.Commarea_Length = commareaLen;
               
               // Return the current date and time in the COMMAREA
               SimpleDateFormat dateFmt = new SimpleDateFormat("dd/MM/yy HH:mm:ss");
               String dateStr = dateFmt.format(new Date());
               byte[] dateBytes = dateStr.getBytes();
               if (commareaLen < dateBytes.length) {
                  throw new RuntimeException("Commarea too small: " + commareaLen);
               }
               System.arraycopy(dateBytes, 0, eciReq.Commarea, 0, dateBytes.length);
               
               return InterceptAction.Return;
            } else if ("EC03".equals(eciReq.Program)) {
               if (eciReq.hasChannel()) {
                  Channel channel = eciReq.getChannel();
                  try {
                     // Get the input data
                     Container inputData;
                     try {
                        inputData = channel.getContainer("INPUTDATA");
                     }  catch (ContainerNotFoundException e) {
                        channel.createContainer("OUTPUTMESSAGE", "INPUTDATA container was not passed to the program");
                        return InterceptAction.Return;
                     }
                     String inputString = inputData.getCHARData();
                     
                     // Create response containers

                     if (inputString.length() > 72) {
                        channel.createContainer("OUTPUTMESSAGE", "INPUTDATA invalid input data length");
                     } else {
                        ByteBuffer inputDataLengthBuffer = ByteBuffer.allocate(4);
                        // ByteBuffer defaults to big endian. To recreate a little endian CICS
                        // system un-comment the next line and the ByteOrder import statement.
   //                     inputDataLengthBuffer.order(ByteOrder.LITTLE_ENDIAN);
                        inputDataLengthBuffer.putInt(inputString.length());
                        channel.createContainer("INPUTDATALENGTH", inputDataLengthBuffer.array());
                        
                        channel.createContainer("OUTPUTMESSAGE", "Input data was: " + inputString);
                     }
                     
                     SimpleDateFormat dateFmt = new SimpleDateFormat("dd/MM/yyyy HH:mm:ss");
                     String dateStr = dateFmt.format(new Date());
                     channel.createContainer("CICSDATETIME", dateStr);
                     
                     // INPUTDATACCSID is normally only returned on CICS TS
                     String inputDataCcsid = Integer.toString(inputData.getCCSID());
                     // Pad with leading spaces
                     inputDataCcsid = "        ".substring(8 - inputDataCcsid.length()) + inputDataCcsid;
                     channel.createContainer("INPUTDATACCSID", inputDataCcsid);
                  } catch (UnsupportedEncodingException e) {
                     throw new RuntimeException(e);
                  } catch (ContainerException e) {
                     throw new RuntimeException(e);
                  }
               } else {
                  // If no Channel passed in, terminate with abend code NOCH
                  eciReq.Cics_Rc = ECIReturnCodes.ECI_ERR_TRANSACTION_ABEND;
                  eciReq.Abend_Code = "NOCH";
               }
               
               return InterceptAction.Return;
            }
         }
      }
      
      return InterceptAction.Continue;
   }
}
