/*
   MODULE NAME       CICSECIX.H

   DESCRIPTIVE NAME  CICS ECI Exit Definitions

 Licensed Materials - Property of IBM  
  
 "Restricted Materials of IBM"  
  
 5724-I81,5725-B65 
  
 (C) Copyright IBM Corp. 1994, 2014  
  
 US Government Users Restricted Rights - Use, duplication or  
 disclosure restricted by GSA ADP Schedule Contract with  
 IBM Corp.  
   
   Status: Version 9 Release 2 

   NOTES :-

   This header file contains definitions and structures for the ECI
   exits.
*/

#ifndef _CICSECIX_H
#define _CICSECIX_H

#include <cicstype.h>
#include <cics_eci.h>

#ifdef __cplusplus
  extern "C" {
#endif

#if defined(CICS_AIX)
#pragma options align=packed
#elif defined(CICS_HPUX)
#pragma pack 1
#else
#pragma pack(1)
#endif

/*
   Special return codes from exits.
   All other non-zero return codes indicate an exit has failed
   but will not cause special handling in the ECI code.
*/
#define CICS_EXIT_OK                     0
#define CICS_EXIT_NO_EXITS               1 /* do not call any other exits */
#define CICS_EXIT_CANT_INIT_EXITS        2 /* Can't initialize exits */
#define CICS_EXIT_GIVE_UP                3 /* exit SystemId retry loop */

/*
   The following values correspond to error cases identified in the
   exits architecture.  They have no special meaning to the ECI
*/
#define CICS_EXIT_BAD_ANCHOR             10 /* Bad anchor value */
#define CICS_EXIT_BAD_STORAGE            11 /* Can't tidy up storage */
#define CICS_EXIT_BAD_PARM               12 /* Bad parameter */

#define CICS_EXIT_USER_BASE             100 /* exit specific codes start here */

/*
  Datatypes
*/
typedef void CICSPTR *CICS_EciAnchor_t;
typedef struct CICS_EciToken
{
    cics_ulong_t Pid;
    cics_ulong_t count;
}   CICS_EciToken_t;

/*
   InitializeExit
   Called after ECI internal initialization is complete
*/
typedef cics_sshort_t CICSEXIT CICS_EciInitializeExitFn
                        (  cics_sshort_t             Version
                        ,  CICS_EciAnchor_t CICSPTR *Anchor
                        );
cics_sshort_t CICSEXIT CICS_EciInitializeExit
                        (  cics_sshort_t             Version
                        ,  CICS_EciAnchor_t CICSPTR *Anchor
                        );

/*
   TerminateExit
   Called before termination
*/
typedef cics_sshort_t CICSEXIT CICS_EciTerminateExitFn
                        ( CICS_EciAnchor_t Anchor
                        );
cics_sshort_t CICSEXIT CICS_EciTerminateExit
                        ( CICS_EciAnchor_t Anchor
                        );

/*
   CICS_EciExternalCallExit1
   CICS_EciExternalCallExit2
*/
typedef cics_sshort_t CICSEXIT CICS_EciExternalCallExit1Fn
                        (    CICS_EciAnchor_t   Anchor
                        ,    CICS_EciToken_t    Token
                        ,    ECI_PARMS CICSPTR *ParmPtr
                        );
cics_sshort_t CICSEXIT CICS_EciExternalCallExit1
                        (    CICS_EciAnchor_t   Anchor
                        ,    CICS_EciToken_t    Token
                        ,    ECI_PARMS CICSPTR *ParmPtr
                        );
typedef cics_sshort_t CICSEXIT CICS_EciExternalCallExit2Fn
                        (    CICS_EciAnchor_t   Anchor
                        ,    CICS_EciToken_t    Token
                        ,    ECI_PARMS CICSPTR *ParmPtr
                        );
cics_sshort_t CICSEXIT CICS_EciExternalCallExit2
                        (    CICS_EciAnchor_t   Anchor
                        ,    CICS_EciToken_t    Token
                        ,    ECI_PARMS CICSPTR *ParmPtr
                        );

/*
   SystemIdExit
*/
typedef cics_sshort_t CICSEXIT CICS_EciSystemIdExitFn
                    (    CICS_EciAnchor_t   Anchor
                    ,    CICS_EciToken_t    Token
                    ,    ECI_PARMS CICSPTR *ParmPtr
                    ,    cics_sshort_t      Reason
                    );
cics_sshort_t CICSEXIT CICS_EciSystemIdExit
                    (    CICS_EciAnchor_t   Anchor
                    ,    CICS_EciToken_t    Token
                    ,    ECI_PARMS CICSPTR *ParmPtr
                    ,    cics_sshort_t      Reason
                    );

/*
   CICS_EciDataSendExit
*/
typedef cics_sshort_t CICSEXIT CICS_EciDataSendExitFn
                        (    CICS_EciAnchor_t Anchor
                        ,    CICS_EciToken_t  Token
                        );
cics_sshort_t CICSEXIT CICS_EciDataSendExit
                        (    CICS_EciAnchor_t Anchor
                        ,    CICS_EciToken_t  Token
                        );

/*
   CICS_EciDataReturnExit
*/
typedef cics_sshort_t CICSEXIT CICS_EciDataReturnExitFn
                        (    CICS_EciAnchor_t   Anchor
                        ,    CICS_EciToken_t    Token
                        ,    ECI_PARMS CICSPTR *ParmPtr
                        );
cics_sshort_t CICSEXIT CICS_EciDataReturnExit
                        (    CICS_EciAnchor_t   Anchor
                        ,    CICS_EciToken_t    Token
                        ,    ECI_PARMS CICSPTR *ParmPtr
                        );

/*
   CICS_EciSetProgramAliasExit
*/
typedef cics_sshort_t CICSEXIT CICS_EciSetProgramAliasExitFn
                        (    CICS_EciAnchor_t   Anchor
                        ,    ECI_PARMS CICSPTR *ParmPtr
                        ,    char              *ProgramName
                        );
cics_sshort_t CICSEXIT CICS_EciSetProgramAliasExit
                        (    CICS_EciAnchor_t   Anchor
                        ,    ECI_PARMS CICSPTR *ParmPtr
                        ,    char              *ProgramName
                        );

typedef struct CICS_EciExitList
{
    CICS_EciInitializeExitFn      CICSPTR *InitializeExit;
    CICS_EciTerminateExitFn       CICSPTR *TerminateExit;
    CICS_EciExternalCallExit1Fn   CICSPTR *ExternalCallExit1;
    CICS_EciExternalCallExit2Fn   CICSPTR *ExternalCallExit2;
    CICS_EciSystemIdExitFn        CICSPTR *SystemIdExit;
    CICS_EciDataSendExitFn        CICSPTR *DataSendExit;
    CICS_EciDataReturnExitFn      CICSPTR *DataReturnExit;
    CICS_EciSetProgramAliasExitFn CICSPTR *SetProgramAliasExit;
}   CICS_EciExitList_t;

#if defined(CICS_AIX)
#pragma options align=reset
#elif defined(CICS_HPUX) && defined(__cplusplus)
#pragma pack
#elif defined(CICS_HPUX) && defined(CICS_HPIT)
#pragma pack
#elif defined(CICS_HPUX)
#pragma HP_ALIGN HPUX_NATURAL
#else
#pragma pack()
#endif

#ifdef __cplusplus
  }
#endif

#endif /* _CICSECIX_H */
