/* Module Name   : ecix2.c

   Product       : CICS Transaction Gateway

   Description   : ECI Exit Library Sample

                   This is a sample which demonstrates ECI exit functionality.
                   It will redirect requests to selected servers.
                   This makes it possible to re-use an existing ECI
                   application with hard-coded server names without
                   having to recompile the application.

                   The following user exit functions are used:
                   CICS_ECIEXITINIT, CICS_EciInitializeExit and
                   CICS_EciExternalCallExit1.
                   The rest of the user exit functions are included
                   due to requirement number 5. below; the code they
                   contain does not perform any actions.

                   When writing exits, the following rules must be obeyed:
                   1. No EPI or ECI calls must be made from the exit
                   2. Exits should contain the minimum possible executable
                      code (to minimize impact on performance)
                   3. All exit code must be re-entrant and thread safe.
                   4. The primary entry point to this module must be named
                      CICS_ECIEXITINIT.  The names of the actual exits may
                      be changed but the parameter lists must not be changed.
                   5. An ECI user exit program should contain valid entry
                      points for all of the user exit functions, with the
                      exception of CICS_EciSetProgramAliasExit which is
                      optional.

 Licensed Materials - Property of IBM  
  
 5724-I81,5725-B65 
  
 (C) Copyright IBM Corp. 2002, 2014 All Rights Reserved.  
  
 US Government Users Restricted Rights - Use, duplication or  
 disclosure restricted by GSA ADP Schedule Contract with  
 IBM Corp.  
     
     Status: Version 9 Release 2 

                   The following code is sample code created by IBM
                   Corporation.  This sample code is not part of any standard
                   IBM product and is provided to you solely for the purpose
                   of assisting you in the development of your applications.
                   The code is provided 'AS IS', without warranty or
                   condition of any kind.  IBM shall not be liable for any
                   damages arising out of your use of the sample code, even
                   if IBM has been advised of the possibility of such damages. */


/* HEADER FILES */
#include <cicstype.h>
#include <cics_eci.h>
#include <cicsecix.h>
#include <string.h>
#include <ctype.h>


void copyPadUpper ( char *destination, char *source );

/* Arrays to define server aliases.

   When an ECI request is made, the user exit looks for a matching
   server name in the redirectFrom array. If it finds one, it
   replaces the eci_system_name parameter with the corresponding
   entry in the redirectTo array. For example, if it matches
   the second element in redirectFrom, eci_system_name is
   replaced by the second element in redirectTo. NUM_SERVERS must
   equal the number of elements of the arrays, and both arrays must
   have the same number of entries.
   If no match is found, the eci_system_name specified by the Client
   application is left unchanged. */

#define NUM_SERVERS 4
char* redirectFrom[NUM_SERVERS] = {"SERVER1", "SERVER2", "SERVER3", "SERVER4"};
char* redirectTo  [NUM_SERVERS] = {"SERVERAA","SERVERAA","SERVERBB","SERVERBB"};



/* The following arrays are used by the user exit code for internal processing.
   The contents of the redirect arrays are padded with spaces, and converted to
   upper case when the user exit is initialised. The internal arrays store the
   results of this. */

char  internalRFrom [NUM_SERVERS][CICS_ECI_SYSTEM_MAX + 1];
char  internalRTo   [NUM_SERVERS][CICS_ECI_SYSTEM_MAX + 1];

/* Main entry point of dynamically loaded object.
   This function returns a structure of addresses of exit functions.
   Note that ALL exits must be defined if any are defined.
   If any exit address is set to NULL no exits will be called. */

#if defined(CICS_W32)
    #define CICSEXPORT __declspec ( dllexport )
#else
    #define CICSEXPORT
#endif

#define copyright_notice copyright4cecix2
char version4cecix2[] =
"\nCICS Transaction Gateway"
" Version 9.2.0.3"
" Build Level c920-20180312-1735" ;

char copyright_notice[] =
"\n"
"Licensed Materials - Property of IBM. "
"\n"
"5724-I81,5725-B65 "
"\n"
"(c) Copyright IBM Corp. 2002, 2014 All Rights Reserved. "
"\n"
"US Government Users Restricted Rights - Use, duplication or "
"disclosure restricted by GSA ADP Schedule Contract with "
"IBM Corp."
"\n";

void CICSEXPORT CICSEXIT CICS_ECIEXITINIT(CICS_EciExitList_t CICSPTR *ExitList)
{
    ExitList->InitializeExit    = (CICS_EciInitializeExitFn CICSPTR *)&CICS_EciInitializeExit;
    ExitList->TerminateExit     = (CICS_EciTerminateExitFn CICSPTR *)&CICS_EciTerminateExit;
    ExitList->ExternalCallExit1 = (CICS_EciExternalCallExit1Fn CICSPTR *)&CICS_EciExternalCallExit1;
    ExitList->ExternalCallExit2 = (CICS_EciExternalCallExit2Fn CICSPTR *)&CICS_EciExternalCallExit2;
    ExitList->SystemIdExit      = (CICS_EciSystemIdExitFn CICSPTR *)&CICS_EciSystemIdExit;
    ExitList->DataSendExit      = (CICS_EciDataSendExitFn CICSPTR *)&CICS_EciDataSendExit;
    ExitList->DataReturnExit    = (CICS_EciDataReturnExitFn CICSPTR *)&CICS_EciDataReturnExit;
}


/* InitializeExit:
   Called immediately after successful initialization.

   Permitted return values:
     CICS_EXIT_OK               No problems - call exits where appropriate
     CICS_EXIT_NO_EXITS         No problems - do not call subsequent exits
                                    (used by shipped dummy library)
     CICS_EXIT_CANT_INIT_EXITS  Exits cannot be used due to an
                                    (unspecified) error condition
     CICS_EXIT_USER_BASE + N    User specified value

   Notes:
     Any value except CICS_EXIT_OK will inhibit the calling of subsequent exits */

cics_sshort_t CICSEXIT CICS_EciInitializeExit ( cics_sshort_t Version,
                                                CICS_EciAnchor_t CICSPTR *Anchor )
{
    int loop;

    /* Make sure that the redirect server names specified are in upper
       case, and trailing white spaces have been removed. EciInitializeExit
       is only called once per process, so this will not entail a significant
       performance overhead. */

    for (loop=0; loop < NUM_SERVERS; loop++) {

        copyPadUpper (internalRTo[loop],redirectTo[loop]);
        copyPadUpper (internalRFrom[loop],redirectFrom[loop]);

    }

    return CICS_EXIT_OK;

}


/* TerminateExit:
   Called immediately prior to the destruction of the
   ECI data structures in the atexit processing.

   Permitted return values:
     CICS_EXIT_OK               No problems
     CICS_EXIT_BAD_ANCHOR       Invalid anchor value supplied
     CICS_EXIT_BAD_STORAGE      Exit cannot tidy up its storage
     CICS_EXIT_USER_BASE + N    User specified value */

cics_sshort_t CICSEXIT CICS_EciTerminateExit( CICS_EciAnchor_t Anchor )
{
    return CICS_EXIT_OK;
}


/* SystemIdExit:
   Called if the ECI would return ECI_ERR_NO_CICS, ECI_ERR_UNKNOWN_SERVER
   or ECI_ERR_SECURITY_ERROR.  This allows the choice of an alternative
   system id.  This is the only exit allowed to request a retry.
   This exit will be called repeatedly until it returns a system which can
   successfully fulfill the request or until it returns CICS_EXIT_OK.
   CICS_EXIT_GIVE_UP can be used to force a non-error return from this loop.

    Permitted return values:
     CICS_EXIT_OK               No problems
     CICS_EXIT_BAD_ANCHOR       Invalid anchor value supplied
     CICS_EXIT_BAD_PARM         One or more parameters bad
     CICS_EXIT_GIVE_UP          There are no more alternative systems
     CICS_EXIT_USER_BASE + N    User specified value */

cics_sshort_t CICSEXIT CICS_EciSystemIdExit ( CICS_EciAnchor_t Anchor,
                                              CICS_EciToken_t Token, ECI_PARMS CICSPTR *ParmPtr,
                                              cics_sshort_t Reason )
{
    return CICS_EXIT_GIVE_UP;
}


/* ExternalCallExit1:
   Called after parameter verification for all LINK and STATUS calls.
   This exit allows system selection.

    Permitted return values:
     CICS_EXIT_OK               No problems
     CICS_EXIT_BAD_ANCHOR       Invalid anchor value supplied
     CICS_EXIT_BAD_PARM         One or more parameters bad
     CICS_EXIT_USER_BASE + N    User specified value */

cics_sshort_t CICSEXIT CICS_EciExternalCallExit1 ( CICS_EciAnchor_t Anchor,
                                                   CICS_EciToken_t Token, ECI_PARMS CICSPTR *ParmPtr )
{

    int loop;
    char tempSystem[CICS_ECI_SYSTEM_MAX + 1];

    /* The CICS Transaction Gateway is case insensitive, and ignores trailing
       spaces when processing server names. By changing the user specified
       system parameter to upper case, and removing trailing spaces, we ensure
       that this user exit code will behave in the same way. */

    copyPadUpper(tempSystem,ParmPtr->eci_system_name);

    /* ParmPtr contains a copy of the ECI parameters the application program specified.
       The for loop below iterates through the redirectFrom array, looking elements
       that match the server name specified by the application. */

    for (loop=0;loop<NUM_SERVERS;loop++) {

        if (!strncmp(tempSystem,internalRFrom[loop],CICS_ECI_SYSTEM_MAX)) {

            /* This redirects the request to the name of the server in
               redirectTo[loop]. The application will be unaware that the request has
               been redirected. User exit code works independantly the language the
               application was written in. */

            memcpy(ParmPtr->eci_system_name, internalRTo[loop],CICS_ECI_SYSTEM_MAX);
            break;
        }
    }

    return CICS_EXIT_OK;

}


/* ExternalCallExit2:
   Called prior to returning from all LINK and STATUS calls.

    Permitted return values:
     CICS_EXIT_OK               No problems
     CICS_EXIT_BAD_ANCHOR       Invalid anchor value supplied
     CICS_EXIT_BAD_PARM         One or more parameters bad
     CICS_EXIT_USER_BASE + N    User specified value */

cics_sshort_t CICSEXIT CICS_EciExternalCallExit2( CICS_EciAnchor_t Anchor,
                                                  CICS_EciToken_t Token,
                                                  ECI_PARMS CICSPTR *ParmPtr)
{
    return CICS_EXIT_OK;
}


/* DataSendExit:
   Called immediately before data is sent to the host.

    Permitted return values:
     CICS_EXIT_OK               No problems
     CICS_EXIT_BAD_ANCHOR       Invalid anchor value supplied
     CICS_EXIT_BAD_PARM         One or more parameters bad
     CICS_EXIT_USER_BASE + N    User specified value */

cics_sshort_t CICSEXIT CICS_EciDataSendExit( CICS_EciAnchor_t Anchor,
                                             CICS_EciToken_t  Token)
{
    return CICS_EXIT_OK;
}


/* DataReturnExit:
   Called immediately after data is received from the host.

    Permitted return values:
     CICS_EXIT_OK               No problems
     CICS_EXIT_BAD_ANCHOR       Invalid anchor value supplied
     CICS_EXIT_BAD_PARM         One or more parameters bad
     CICS_EXIT_USER_BASE + N    User specified value */

cics_sshort_t CICSEXIT CICS_EciDataReturnExit( CICS_EciAnchor_t Anchor,
                                               CICS_EciToken_t Token,
                                               ECI_PARMS CICSPTR *ParmPtr)
{
    return CICS_EXIT_OK;
}


/* copyPadUpper: an internal function used by the sample user exit code.

   Copies a string, ensuring that the copied string is:
   8 characters in length, padded with trailing nulls and upper case

   The area of memory pointed to by 'destination' must be 8 bytes long. */

void copyPadUpper ( char* destination, char* source ) {

    int loop;
    char Ch;

    /* Copy at most CICS_ECI_SYSTEM_MAX characters across. */

    strncpy(destination,source,CICS_ECI_SYSTEM_MAX);
    destination[CICS_ECI_SYSTEM_MAX] = 0;

    /* Remove any trailing spaces. */

    loop=CICS_ECI_SYSTEM_MAX;
    while (loop>=0 && (destination[loop]==' ' || destination[loop]==0) ) {
        destination[loop]=0;
        loop--;
    }

    /* Convert to upper case */

    for (loop=0;loop<CICS_ECI_SYSTEM_MAX;loop++) {
        Ch = toupper(destination[loop]);
        destination[loop] = Ch;
    }

}
