/* File Name(s)  : ctgstat1.c

   Product       : CICS Transaction Gateway

   Description   : This sample shows the basic use of the CICS
                   statistics API. It demonstrates calls to the CICS
                   Transaction Gateway statistics DLL module to
                   connect to a Gateway daemon, obtain ID result sets,
                   generate query strings from ID result sets and
                   obtain statistics result sets.

   Prerequisites:  This sample requires the CICS Transaction Gateway
                   statistics DLL to be installed.

 Licensed Materials - Property of IBM  
  
 5724-I81,5725-B65,5655-Y20 
  
 (C) Copyright IBM Corp. 2006, 2014 All Rights Reserved.  
  
 US Government Users Restricted Rights - Use, duplication or  
 disclosure restricted by GSA ADP Schedule Contract with  
 IBM Corp.  
   
   Status: Version 9 Release 2 

                   The following code is sample code created by IBM
                   Corporation. This sample code is not part of any
                   standard IBM product and is provided to you solely
                   for the purpose of assisting you in the development
                   of your applications. The code is provided 'AS IS',
                   without warranty or condition of any kind. IBM shall
                   not be liable for any damages arising out of your
                   use of the sample code, even if IBM has been advised
                   of the possibility of such damages. */
#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include "ctgstats.h"
#include "ctgstdat.h"

#define copyright_notice copyright4cctgstat1
char version4cctgstat1[] =
"\nCICS Transaction Gateway"
" Version 9.2.0.3"
" Build Level c920-20180312-1735" ;

char copyright_notice[] =
"\n"
"Licensed Materials - Property of IBM. "
"\n"
"5724-I81,5725-B65,5655-Y20 "
"\n"
"(c) Copyright IBM Corp. 2006, 2014 All Rights Reserved. "
"\n"
"US Government Users Restricted Rights - Use, duplication or "
"disclosure restricted by GSA ADP Schedule Contract with "
"IBM Corp."
"\n";

int checkProtocolVersion(char *gwProtocolVer);

int main(int argc, char **argv){
   /*Gateway token representing the connection*/
   CTG_GatewayToken_t gatewayToken;     
   /*Result Set token for results from statistics calls*/
   CTG_ResultSetToken_t resultSetToken; 
   /*Struct to hold individual Stat information*/
   CTG_StatData_t statData;             
   /*The return code from statistics calls*/
   int statsRc;                         
   /*API protocol version from the Gateway*/
   char *protocolVersion;               
   /*Resource group query string*/
   char *queryString1;                  
   /*Statistic IDs query string*/
   char *queryString2;                  
   /*Data on how long the Gateway daemon has been running*/
   long uptime, seconds, minutes, hours;
   /*The number of requests the Gateway daemon has processed*/
   int numberOfRequests;
   /*Storage for the Query Element from the result set*/
   char queryElement[256] = {0};                
   /*Storage for the Stat Id from the result set*/
   char statName[256] = {0};            
   /*Storage for the Stat Value from the result set*/
   char statValue[256] = {0};           
   /*Return code for the application*/
   int rc = 0;                          
   /*The port the Gateway daemon is listening on*/     
   int port = 2980;                     
   /*Flag requirement to disconnect from the Gateway daemon*/
   int connected = 0;  
   /*Gateway host name*/
   char *hostname;
   /*Flag for whether to use a remote Gateway connection*/
   int useRemote;              
      
   /*Display banner and usage information*/
   printf("CICS Transaction Gateway Statistics Sample\n\n");
   printf("Usage: ctgstat1 [<host name> <port number>] | [<port number>]\n\n");
   
   /*Set a flag for using a remote Gateway connection, based on whether both a
     host name and port were specified in the command line arguments*/
   if(argc > 2){
      useRemote = 1;
   } else {
      useRemote = 0;
   }

   /*Parse command line arguments*/
   /*If using a remote Gateway connection, retrieve the host name and port
     number*/
   if(useRemote == 1){
      hostname = argv[1];
      port = atoi(argv[2]);
   } else {
      /*If using a local connection, retrieve the port number if specified*/
      if(argc == 2){
         port = atoi(argv[1]);
      }
   }
   
   /*Check the port number is valid*/
   if((port < 1) || (port > 65536)){
      printf("Error: invalid port number specified. Using default port "
         "2980.\n\n");
      port = 2980;
   }

   /*Open a connection to the Gateway. If the host name and port have been
     specified, then open a remote connection*/
   if(useRemote == 1){
      statsRc = openRemoteGatewayConnection(hostname, port, &gatewayToken,
         &protocolVersion);
   } else {
      statsRc = openGatewayConnection(port, &gatewayToken, &protocolVersion);
   }
   
   if(statsRc != CTG_STAT_OK){
      if(useRemote == 1){
         printf("Failed to connect to remote Gateway daemon on host name %s "
            "port %d. rc=%d\n\n", hostname, port, statsRc);
      } else {
         printf("Failed to connect to local Gateway daemon on port %d. "
            "rc=%d\n\n", port, statsRc);
      }
      rc = -1;
   } else {
       /*Set flag for disconnect later*/
       connected = 1; 
   }
   
   if(rc == 0){
      if(useRemote == 1){
         printf("Successfully connected to remote Gateway daemon on host name "
            "%s port %d\n\n", hostname, port);
      } else {
         printf("Successfully connected to local Gateway daemon on port %d\n\n",
            port);
      }
      
      statsRc = checkProtocolVersion(protocolVersion);
      if(statsRc != 0){
         printf("Protocol mismatch: Gateway = %s, ctgstdat = %s\n\n", 
            protocolVersion, CTG_STAT_PROTOCOL_VER);
         rc = -2;
      }
   }
   
   if(rc == 0){      
      /*Get all the IDs of the statistics in the CM resource group*/
      queryString1 = "CM";
      statsRc = getStatIdsByStatGroupId(gatewayToken, queryString1, 
         &resultSetToken);
      if(statsRc != CTG_STAT_OK){
         printf("Error getting Stat Ids from the Gateway daemon. rc=%d\n\n", 
            statsRc);
         rc = -3;
      }
   }
   
   if(rc == 0){
      /*Get a query string based on the results of the previous call*/
      statsRc = getIdQuery(resultSetToken, &queryString2);
      if(statsRc != CTG_STAT_OK){
         printf("Unable to generate query string from result set. "
            "rc=%d\n\n", statsRc);
         rc = -4;
      }
   }
   
   if(rc == 0){
      printf("Statistics from %s resource group to collect: %s\n\n", 
         queryString1, queryString2);
      /*Get the values of all the statistics in the CM resource group*/
      statsRc = getStatsByStatId(gatewayToken, queryString2, 
         &resultSetToken);
      if(statsRc != CTG_STAT_OK){      
         printf("Unable to query the Gateway daemon for statistics. rc=%d\n\n",
             statsRc);
         rc = -5;
      }
   }
   
   if(rc == 0){
      printf("Statistics values:\n");
      /*Iterate over the result set and display the statistics*/
      statsRc = getFirstStat(resultSetToken, &statData);
      while(statsRc == CTG_STAT_OK){
         if(statData.queryElementRC == CTGSTATS_SUCCESSFUL_QUERY){
            strncpy(statName, statData.statIdPtr, statData.statIdLen);
            statName[statData.statIdLen] = '\0';
            strncpy(statValue, statData.statValuePtr, 
                    statData.statValueLen);
            statValue[statData.statValueLen] = '\0';
            printf("\t%s=%s\n", statName, statValue);
         } else {
            strncpy(queryElement, statData.queryElementPtr,
                    statData.queryElementLen);
            queryElement[statData.queryElementLen] = '\0';
            printf("\tQuery for %s failed with rc %d\n", queryElement,
                   statData.queryElementRC);
         }
         statsRc = getNextStat(resultSetToken, &statData);
      }
      if(statsRc != CTG_STAT_OK && 
         statsRc != CTG_STAT_ERR_ENDOFRESULTSET){
         printf("Error when parsing the result set. rc=%d\n\n", statsRc);
         rc = -6;
      } else {
         printf("End of results\n\n");
      }
   }
   
   if(rc == 0){
      /*Get the running time statistic*/
      statsRc = getStatsByStatId(gatewayToken, "GD_LRUNTIME", 
         &resultSetToken);
      if(statsRc != CTG_STAT_OK){      
         printf("Unable to query the Gateway daemon for statistics. rc=%d\n\n", 
            statsRc);
         rc = -7;
      } else {
         /*Calculate the uptime in hours, minutes and seconds*/
         statsRc = getFirstStat(resultSetToken, &statData);
         if(statsRc == CTG_STAT_OK){
            if(statData.queryElementRC == CTGSTATS_SUCCESSFUL_QUERY){
               strncpy(statValue, statData.statValuePtr, 
                       statData.statValueLen);
               statValue[statData.statValueLen] = '\0';
               uptime = atol(statValue);
               minutes = (long)uptime / 60;
               seconds = uptime % 60;
               hours = (long)minutes / 60;
               minutes = minutes % 60;
               printf("Gateway daemon uptime = %ld hrs %ld mins %ld secs\n\n", 
                  hours, minutes, seconds);
            } else {
               strncpy(queryElement, statData.queryElementPtr,
                       statData.queryElementLen);
               queryElement[statData.queryElementLen] = '\0';
               printf("Query for %s failed with rc %d\n\n", 
                      queryElement, statData.queryElementRC);
               rc=-8;
            }
         } else {
            printf("Unable to get statistics data from result set. rc=%d\n",
                   statsRc);
         }
      }       
   }
   
   if(rc == 0){
      /*Get the all requests statistic*/
      statsRc = getStatsByStatId(gatewayToken, "GD_LALLREQ", 
         &resultSetToken);
      if(statsRc != CTG_STAT_OK){      
         printf("Unable to query the Gateway daemon for statistics. rc=%d\n\n", 
            statsRc);
         rc = -9;
      } else {
         statsRc = getFirstStat(resultSetToken, &statData);
         if(statsRc == CTG_STAT_OK){
            if(statData.queryElementRC == CTGSTATS_SUCCESSFUL_QUERY){
               strncpy(statValue, statData.statValuePtr, 
                       statData.statValueLen);
               statValue[statData.statValueLen] = '\0';
               numberOfRequests = atoi(statValue);
               printf("Gateway daemon has processed %.2f requests per second\n\n", 
                  (double)numberOfRequests/(double)uptime);
            } else {
               strncpy(queryElement, statData.queryElementPtr,
                       statData.queryElementLen);
               queryElement[statData.queryElementLen] = '\0';
               printf("Query for %s failed with rc %d\n\n", 
                      queryElement, statData.queryElementRC);
            }
         } else {
            printf("Unable to get statistics data from result set. rc=%d\n",
                   statsRc);
         }
      }
   }

   if (connected == 1) {
       /*Close the connection to the Gateway daemon*/
       statsRc = closeGatewayConnection(&gatewayToken);
       if(statsRc != CTG_STAT_OK){
          printf("Failed to close connection to Gateway daemon. "
            "rc=%d\n\n", statsRc);
          rc = -10;
       } else {
           /*Log successful disconnection*/
           printf("Successfully disconnected from Gateway daemon\n\n");
       }

   }

   if(rc == 0){
      printf("ctgstat1 completed successfully\n");
   } else {
      printf("ctgstat1 encountered errors during execution\n");
   }
   
   return rc;
}

/* Compare the protocol version returned from the Gateway daemon with the 
 * value in ctgstdat.h.
 * 
 * If the Gateway version is greater then return -1 else return 0.
 */
int checkProtocolVersion(char *gwProtocolVer){
   char *localProtocolVer = CTG_STAT_PROTOCOL_VER;
   
   char gwMajorVer[2] = {gwProtocolVer[0], '\0'};
   char localMajorVer[2] = {localProtocolVer[0], '\0'};
   
   if(atoi(gwMajorVer) > atoi(localMajorVer)){
      return -1;
   } else {
      return 0;
   }
}
